//
//  ivAppObjAlgoDebugMotion.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by sulaispig on 7/19/15.
//  Copyright (c) 2015 sulaispig. All rights reserved.
//

#include "ivAppObjAlgoDebugMotion.h"

inline void visualizeMotionField(ivAppObjectFrameData &ofd, bool isPatch, bool isFirstFrame)
{
    cv::Mat &motionField = isPatch ? ofd.objectMotionField : ofd.shadowMotionField;

    cv::Mat &motionFieldVis = isPatch ? ofd.objectMotionVis : ofd.shadowMotionVis;
    motionFieldVis = cv::Mat( OSZ_HEIGHT, OSZ_WIDTH, CV_8UC3, cv::Scalar::all(255) );

    if( !isFirstFrame )
    {
        Utils_Ocv &uocv = Utils_Ocv::instance();
        uocv.motionToColor( motionField, motionFieldVis );
    }
    
    const cv::Mat &srcPatch = isPatch ? ofd.objPatch : ofd.shadowPatch;
    for(int pi=0;pi<OSZ_HEIGHT;pi++) for(int pj=0;pj<OSZ_WIDTH;pj++)
    {
        const cv::Vec4b &spp = srcPatch.at<cv::Vec4b>(pi,pj);
        if( spp[3]>0 )
            continue;
        
        cv::Vec3b &mfv = motionFieldVis.at<cv::Vec3b>(pi,pj);
        mfv[0] = mfv[1] = mfv[2] = 255;
    }
}

bool ivAppObjAlgoDebugObjectMotion::execute(ivAppObject *obj)
{
    ivAppObjectFrameData &ofd = obj->objData.frames.back();
    visualizeMotionField( ofd, true, obj->objData.frames.size()<=1 );
    return true;
}

bool ivAppObjAlgoDebugShadowMotion::execute(ivAppObject *obj)
{
    ivAppObjectFrameData &ofd = obj->objData.frames.back();
    visualizeMotionField( ofd, false, obj->objData.frames.size()<=1 );
    return true;
}