//
//  ivAppObjAlgoDebugWarps.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by sulaispig on 7/19/15.
//  Copyright (c) 2015 sulaispig. All rights reserved.
//

#include "ivAppObjAlgoDebugWarps.h"

#define ERROR_BOOST 255

inline cv::Scalar getColorbyWeight(float weight, bool R, bool G, bool B)
{
    weight = std::min( 1.0f, weight );
    weight = std::max( 0.0f, weight );
    int value = weight * 255;
    
    return cv::Scalar( value*B, value*G, value*R );
}

inline void debugWarps(ivAppObject *obj, bool isPatch)
{
    ivAppObjectData &objData = obj->objData;
    int clusterNum = (int)objData.parts.size();
    
    ivAppObjectFrameData &objFrameData = objData.frames.back();
    
    patchCollager &collager = isPatch ? objFrameData.objectCollager : objFrameData.shadowCollager;
    
    //visualize warps
    for(int i=0;i<clusterNum;i++)
    {
        ObjShdWarpData &oswData = objFrameData[i];
        WarpData &wData = isPatch ? oswData.object : oswData.shadow;
        collager.getSrcIDWeightMap( wData.debugWeightMap, i );
    }
    
    int rowNum = 3;
    int colNum = std::ceil((float)clusterNum/rowNum);
    cv::Size wOsz( colNum*OSZ_WIDTH, rowNum*OSZ_HEIGHT );
    
    cv::Mat &warpClgs = isPatch ? objFrameData.objectWarps : objFrameData.shadowWarps;
    warpClgs = cv::Mat( wOsz, CV_8UC3, cv::Scalar::all(0) );
    
    //the roi images for clusters
    std::vector<cv::Mat> roiImages( clusterNum );
    for(int i=0;i<clusterNum;i++)
    {
        int rID = i%3;
        int cID = i/3;
        roiImages[i] = warpClgs( cv::Rect( cID*OSZ_WIDTH, rID*OSZ_HEIGHT, OSZ_WIDTH, OSZ_HEIGHT ) );
    }
    
    int usedClusterID=0;
    for(int i=0;i<clusterNum;i++)
    {
        ObjShdWarpData &oswData = objFrameData[i];
        
        //get colored object patch
        WarpData &wData = isPatch ? oswData.object : oswData.shadow;
        if( wData.isReadyForWarping )
        {
            cv::Mat img = collager.getColoredImg(usedClusterID, 0.5f);
            cv::cvtColor(img, img, CV_RGBA2RGB, 3);
            
            const cv::Mat &warpWeightMap = wData.warpWeightMap;
            for(int gi=0;gi<OSZ_HEIGHT;gi++) for(int gj=0;gj<OSZ_WIDTH;gj++)
            {
                const float &imp = warpWeightMap.at<float>(gi,gj);
                cv::Vec3b &color = img.at<cv::Vec3b>(gi,gj);
                
                Utils_Ocv::instance().getColorBGR( color, ERROR_BOOST*imp, 255, 0 );
            }
            
//            NearestFrameData &pfds = wData.nfData;
//            int nframeID = pfds.nframeID;
            
//            std::ostringstream oss;
//            oss << "(frame, weight, error): ("
//            << nframeID << ", " << wData.warpWeight << ", " <<  wData.warpError << ")";
//            cv::putText( img, oss.str(), cv::Point(40,40),  FONT_HERSHEY_COMPLEX , 0.5, cv::Scalar::all(255) );
            
            img.copyTo( roiImages[i] );
            
            usedClusterID++;
        }
        
//        //visualize in-cluster points
//        for(int j=0;j<wData.oriPtNum;j++)
//        {
//            const FrontPoint &fp = wData.frontPoints[j];
//            cv::Scalar color = getColorbyWeight( fp.pWeight, true, true, true );
//            cv::circle( roiImages[i], fp.toPt, 3, color, -1 );
//        }
//        
//        //visualize  edges
//        std::vector<FrontEdge>::iterator eit = wData.frontEdges.begin();
//        for( ; eit!=wData.frontEdges.end(); eit++ )
//        {
//            const Eigen::Vector2i &ftvptIDs = eit->ftvptIDs;
//            FrontPoint &fp0 = wData.frontTvPoints[ ftvptIDs(0) ];
//            FrontPoint &fp1 = wData.frontTvPoints[ ftvptIDs(1) ];
//            cv::line( roiImages[i], fp0.toPt, fp1.toPt, cv::Scalar::all(255) );
//        }
    }
    
//    //visualize shared points
//    const int sfmPtNum = (int)obj->objSfM.vPointData.size();
//    for( int i=0; i<sfmPtNum; i++ )
//    {
//        //the cluster pair where the clusters are ready for warping
//        for( int j=0; j<clusterNum; j++ )
//            for( int k=j+1; k<clusterNum; k++ )
//            {
//                const WarpData &wDataJ = isPatch ? objFrameData[j].object : objFrameData[j].shadow;
//                if( !wDataJ.isReadyForWarping )
//                    continue;
//                
//                const WarpData &wDataK = isPatch ? objFrameData[k].object : objFrameData[k].shadow;
//                if( !wDataK.isReadyForWarping )
//                    continue;
//                
//                int fpid_j = wDataJ.sfm2fp[i];
//                int fpid_k = wDataK.sfm2fp[i];
//                
//                //the point is not in one of the clusters
//                if( fpid_j<0 || fpid_k<0 )
//                    continue;
//                
//                const FrontPoint &fpJ = wDataJ.frontPoints[fpid_j];
//                cv::Scalar colorJ = getColorbyWeight( fpJ.pWeight, true, true, true );
//                cv::circle( roiImages[j], fpJ.toPt, 5, colorJ );
//                
//                const FrontPoint &fpK = wDataK.frontPoints[fpid_k];
//                cv::Scalar colorK = getColorbyWeight( fpK.pWeight, true, true, true );
//                cv::circle( roiImages[k], fpK.toPt, 5, colorK );
//            }
//    }
//    
//    //visualize shared edge end-points
//    const int tvptNum = (int)obj->objTvEdges.vtvPoint.size();
//    for( int i=0; i<tvptNum; i++ )
//    {
//        //the cluster pair where the clusters are ready for warping
//        for( int j=0; j<clusterNum; j++ )
//            for( int k=j+1; k<clusterNum; k++ )
//            {
//                const WarpData &wDataJ = isPatch ? objFrameData[j].object : objFrameData[j].shadow;
//                if( !wDataJ.isReadyForWarping )
//                    continue;
//                
//                const WarpData &wDataK = isPatch ? objFrameData[k].object : objFrameData[k].shadow;
//                if( !wDataK.isReadyForWarping )
//                    continue;
//                
//                int ftvpid_j = wDataJ.tvp2ftvp[i];
//                int ftvpid_k = wDataK.tvp2ftvp[i];
//                
//                //the end point is not in one of the clusters
//                if( ftvpid_j<0 || ftvpid_k<0 )
//                    continue;
//                
//                const FrontPoint &ftvpJ = wDataJ.frontTvPoints[ftvpid_j];
//                cv::Scalar colorJ = getColorbyWeight( ftvpJ.pWeight, true, true, true );
//                cv::circle( roiImages[j], ftvpJ.toPt, 5, colorJ );
//                
//                const FrontPoint &ftvpK = wDataK.frontTvPoints[ftvpid_k];
//                cv::Scalar colorK = getColorbyWeight( ftvpK.pWeight, true, true, true );
//                cv::circle( roiImages[k], ftvpK.toPt, 5, colorK );
//            }
//    }
}


bool ivAppObjAlgoDebugObjectWarps::execute(ivAppObject *obj)
{
    debugWarps(obj,true);
    return true;
}


bool ivAppObjAlgoDebugShadowWarps::execute(ivAppObject *obj)
{
    debugWarps(obj,false);
    return true;
}
