//
//  ivAppObjAlgoGEPWarping.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by sulaispig on 3/5/15.
//  Copyright (c) 2015 sulaispig. All rights reserved.
//

#include "ivAppObjAlgoGEPWarping.h"

inline void debugWarping(ivAppObject *obj)
{
    //0:frame, 1:edge
    int type = 0;
    
    //cluster id
    int cid = 0;
    
    int clusterNum = (int)obj->objData.size();
    
    while( true )
    {
        cv::Mat objectPatch;
        cv::Mat shadowPatch;
        if( type == 0 )
        {
            objectPatch = obj->objData[cid].object.warpedFrame;
            shadowPatch = obj->objData[cid].shadow.warpedFrame;
        }
        if( type == 1 )
        {
            objectPatch = obj->objData[cid].object.warpedEdgeMap;
            shadowPatch = obj->objData[cid].shadow.warpedEdgeMap;
        }
        
        cv::imshow("objectPatch", objectPatch*obj->objData[cid].object.isWarpingOK );
        cv::imshow("shadowPatch", shadowPatch*obj->objData[cid].shadow.isWarpingOK );
        
        char ch = cv::waitKey(1);
        if(ch=='q')break;
        
        if( ch=='a' )
            cid = (cid+1)%clusterNum;
        
        if( ch=='d' )
            type = (type+1)%2;
    }
}

bool ivAppObjAlgoGEPWarpingObjectPatch::simpleWarp(WarpData &wData, const ivAppObject *obj)
{
    if( !wData.isWarpingOK )
        return false;
    
    const ivAppObjParams &params = obj->objParams;
    
    //the gradient edge preserving warper
    GEPWarper warper( wData.warpedFrame, params.WRP_gridSize );
    
    //step1: positional constraints
    int ptNum = (int)wData.fromPts.size();
    for(int i=0;i<ptNum;i++)
        warper.addPointConstraint( wData.fromPts[i], wData.toPts[i] );
    
    //step2: edge contraints
    std::vector<FrontEdge> &frontEdges = wData.frontEdges;
    for( std::vector<FrontEdge>::iterator it = frontEdges.begin(); it!=frontEdges.end(); it++ )
        warper.addEdgeConstraint( it->fromPt0, it->fromPt1, params.WRP_eintrvl, it->alpha, it->b );
    
    //go warping
    cv::Size osz = cv::Size(OSZ_WIDTH, OSZ_HEIGHT);
    warper.goWarping( params.WRP_simW, params.WRP_edgeW, params.WRP_udiff );
    
    //copy results
    wData.warpedFrame = warper.getDeformedImg( osz, params.WRP_rgrid );
    wData.warpedEdgeMap = warper.getDeformedImg( wData.warpedEdgeMap, osz, params.WRP_rgrid );
    wData.warpedEdgeCornerMap = warper.getDeformedImg( wData.warpedEdgeCornerMap, osz, params.WRP_rgrid );
    
    //remap the points(edges) to warped space
    for(int i=0;i<ptNum;i++)
        warper.remapPoint( wData.fromPts[i], wData.toPts[i] );
    for( std::vector<FrontEdge>::iterator it = frontEdges.begin(); it!=frontEdges.end(); it++ )
    {
        warper.remapPoint( it->fromPt0, it->toPt0 );
        warper.remapPoint( it->fromPt1, it->toPt1 );
    }
    
    return true;
}

bool ivAppObjAlgoGEPWarpingObjectPatch::execute(ivAppObject *obj)
{
    ivAppObjectData &objData = obj->objData;
    int clusterNum = (int)objData.size();
    for(int i=0;i<clusterNum;i++)
        //warp object patch
        this->simpleWarp( objData[i].object, obj );
    
    return true;
}

bool ivAppObjAlgoGEPWarpingShadowPatch::execute(ivAppObject *obj)
{
    ivAppObjectData &objData = obj->objData;
    int clusterNum = (int)objData.size();
    for(int i=0;i<clusterNum;i++)
        //warp shadow patch
        this->simpleWarp( objData[i].shadow, obj );
    
    return true;
}

