//
//  ivAppObjAlgoEdgeClassification.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjAlgoEdgeClassification.h"

bool ivAppObjAlgoEdgeClassification::execute(ivAppObject *obj)
{
    std::vector<ivAppObjectPart> &objParts = obj->objData.parts;
    float CLSTR_edgeDist = obj->objParams.CLSTR_edgeDist;
    float CLSTR_edgeOrnt = std::cos( M_PI/2.0f - obj->objParams.CLSTR_edgeOrnt * M_PI / 180.0f );
    
    //for each edge
    const ivAppTvEdges &objTvEdges = obj->objTvEdges;
    const std::vector<tvEdge> &vtvEdge = objTvEdges.vtvEdge;
    std::vector<tvEdge>::const_iterator eit = vtvEdge.begin();
    for( ; eit!=vtvEdge.end(); eit++ )
    {
        int v1 = eit->vids(0);
        int v2 = eit->vids(1);
        
        //for each cluster
        std::vector<ivAppObjectPart>::iterator cit = objParts.begin();
        for(;cit!=objParts.end();cit++)
        {
            Plane &p = cit->tanPlane;

            //get plane normal
            Eigen::Vector3f pNormal;
            p.getNormal(pNormal);

            //to-plane error
            const Eigen::Vector4f &pt3d1 = objTvEdges.vtvPoint[v1].v3D;
            const Eigen::Vector4f &pt3d2 = objTvEdges.vtvPoint[v2].v3D;
            
            float dist = 0.0f;
            dist += p.getShortestDist(pt3d1);
            dist += p.getShortestDist(pt3d2);
            
            float odiff = std::abs( (pt3d1-pt3d2).block(0,0,3,1).dot( pNormal ) );

            //the edge belongs to the cluster
            if( dist < CLSTR_edgeDist && odiff < CLSTR_edgeOrnt )
            {
                int eIndex = (int)std::distance( vtvEdge.begin(), eit );
                cit->edgeIDs.push_back(eIndex);
            }
        }
    }
    
    return true;
}

