//
//  ivAppObjAlgoCheckFrames.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjAlgoCheckFrames.h"

bool ivAppObjAlgoCheckFrame::ifObjShdFramesValid(bool isPatch, const ivAppObject *obj)
{
    const ivAppObjectData &objData = obj->objData;
    const ivAppObjectFrameData &objFrameData = objData.frames.back();
    const int clusterNum = (int)objData.parts.size();
    
    //object sim graph
    const ivAppObjectSfMSimGraph &objSimGraph = obj->objSimGraph;
    
    bool isFramesValid = true;
    for(int i=0;i<clusterNum;i++)for(int j=i+1;j<clusterNum;j++)
    {
        const NearestFrameData &nfesi = isPatch ? objFrameData[i].object.nfData : objFrameData[i].shadow.nfData;
        const NearestFrameData &nfesj = isPatch ? objFrameData[j].object.nfData : objFrameData[j].shadow.nfData;
        
        bool isValidEdge = objSimGraph.isNeighbor( nfesi.nframeID, nfesj.nframeID );
        isFramesValid = isFramesValid && isValidEdge;
    }
    
    return isFramesValid;
}

bool ivAppObjAlgoCheckFrame::execute(ivAppObject *obj)
{
    ivAppObjectData &objData = obj->objData;
    ivAppObjectFrameData &objFrameData = objData.frames.back();
    objFrameData.ifObjectPoseFramesValid = this->ifObjShdFramesValid(true, obj);
    objFrameData.ifShadowPoseFramesValid = this->ifObjShdFramesValid(false, obj);
    
    return true;
}


void ivAppObjAlgoCheckFrames::ifObjShdFramesValid
( std::vector<bool> &ifMotionFramesValid, bool isPatch, const ivAppObject *obj )
{
    const ivAppObjectData &objData = obj->objData;
    const int clusterNum = (int)objData.parts.size();

    const ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    const int aniPosNum = guiParams.aniMaxPos+1;
    
    //object sim graph
    const ivAppObjectSfMSimGraph &objSimGraph = obj->objSimGraph;
    
    //selected frames
    const Eigen::Matrix<NearestFrameData,Eigen::Dynamic,Eigen::Dynamic> &nframes = isPatch ?
    objData.objNFrames : objData.shdNFrames;

    //check results
    ifMotionFramesValid = std::vector<bool>( aniPosNum, true );
    
    //check the temporal edges
    for(int i=0;i<clusterNum;i++) for(int j=0;j<aniPosNum-1;j++)
    {
        int k=j+1;
        const NearestFrameData &nfdj = nframes(i,j);
        const NearestFrameData &nfdk = nframes(i,k);
        
        bool isValidEdge = objSimGraph.isNeighbor( nfdj.nframeID, nfdk.nframeID );
        ifMotionFramesValid[j] = ifMotionFramesValid[j] && isValidEdge;
        ifMotionFramesValid[k] = ifMotionFramesValid[k] && isValidEdge;
    }

    //check spatial edges
    for(int i=0;i<clusterNum;i++)for(int j=i+1;j<clusterNum;j++)
    {
        for(int k=0;k<aniPosNum;k++)
        {
            const NearestFrameData &nfeik = nframes(i,k);
            const NearestFrameData &nfejk = nframes(j,k);
            
            bool isValidEdge = objSimGraph.isNeighbor( nfeik.nframeID, nfejk.nframeID );
            ifMotionFramesValid[k] = ifMotionFramesValid[k] && isValidEdge;
        }
    }
}


bool ivAppObjAlgoCheckFrames::execute(ivAppObject *obj)
{
    ivAppObjectData &objData = obj->objData;
    this->ifObjShdFramesValid( objData.ifObjectMotionFramesValid,  true, obj );
    this->ifObjShdFramesValid( objData.ifShadowMotionFramesValid, false, obj );
    
    return true;
}
