//
//  ivAppObjAlgorithm.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjAlgorithm.h"

std::string ivAppObjAlgorithm::toString()
{
    return typeid(*this).name();
}

//execute all the process in the list
bool ivAppObjAlgorithmList::execute(ivAppObject *obj, std::string processName, bool useTimer)
{
    if( !obj->isReady )
        return false;
    
    Utils_Timer timer;
    
    if(useTimer)
        timer.start( processName );
    
    //start the algorithm
    std::list< ivAppObjAlgorithm* >::iterator ait = this->begin();
    for( ; ait!= this->end(); ait++ )
    {
        if( useTimer )
            timer.start( (*ait)->toString() );

        if( !(*ait)->execute(obj) )
        {
            std::cout << (*ait)->toString() << " fails." << std::endl;
            break;
        }
        
        if( useTimer )
            timer.stop( (*ait)->toString() );
    }

    if(useTimer)
    {
        timer.stop( processName );
        std::cout << std::endl;
    }
    
    return true;
}

void ivAppObjAlgorithmList::release()
{
    std::list< ivAppObjAlgorithm* >::iterator ait = this->begin();
    for( ; ait!= this->end(); ait++ )
        delete *ait;
    this->clear();
}
