//
//  IndexedCircularList.hpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__IndexedCircularList__
#define __ivManipulation__IndexedCircularList__

template <class OBJ>
class IndexedCircularList
{
protected:
    //object linked lists
    std::list<OBJ> ivaObjList;
    
    //pointer to the current object
    typename std::list<OBJ>::iterator pCurObject;
    
public:

    //constructor
    IndexedCircularList()
    {
        this->clear();
    }
    
    //if the list is empty
    bool empty() const
    {
        return ivaObjList.empty();
    }
    
    int size() const
    {
        return (int)ivaObjList.size();
    }
    
    //rm all objects
    void clear()
    {
        this->ivaObjList.clear();
        this->pCurObject = this->ivaObjList.begin();
    }
    
    //get current object
    OBJ* getCurrentObject()
    {
        if( ivaObjList.size()>0 )
            return &(*pCurObject);
        
        return 0;
    }
    
    //move the index to position objID
    void pointToObjAt( int objID )
    {
        if( objID >= 0 && objID<size() )
        {
            this->pCurObject = ivaObjList.begin();
            std::advance( this->pCurObject, objID);
        }
    }
    
    //get the current position
    int getCurPos()const 
    {
        if( empty() )
            return -1;
        
        typename std::list<OBJ>::const_iterator bgn = ivaObjList.begin();
        typename std::list<OBJ>::const_iterator pCur = pCurObject;
        
        return (int)std::distance( bgn, pCur );
    }

    //add the object to the list
    void addObject(OBJ &obj)
    {
        if( !ivaObjList.empty() )
            this->pCurObject++;
        
        this->ivaObjList.insert( this->pCurObject, obj );
        this->pCurObject--;
    }
    
    //duplicate
    void duplicateObject()
    {
        if( !ivaObjList.empty() )
            this->ivaObjList.insert( this->pCurObject, *this->pCurObject );
    }
    
    //remove current object from the list
    void rmCurObject()
    {
        if( ivaObjList.empty() )
            return;
        
        this->pCurObject = this->ivaObjList.erase( this->pCurObject );
        if( this->pCurObject == this->ivaObjList.end() )
            this->pCurObject--;
    }
    
    //goto the next object
    void nextObject()
    {
        if( ivaObjList.empty() )
            return;
        
        this->pCurObject++;
        if( this->pCurObject == this->ivaObjList.end() )
            this->pCurObject = this->ivaObjList.begin();
    }

    //goto the previous object
    void prevObject()
    {
        if( ivaObjList.empty() )
            return;
        
        if( this->pCurObject == this->ivaObjList.begin() )
            this->pCurObject = this->ivaObjList.end();
            this->pCurObject--;
    }

};

#endif
