//
//  ivAppObjAlgoMain.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjAlgoMain.h"

ivAppObjAlgoMain ivAppObjAlgoMain::inst;

//execute the process
bool ivAppObjAlgoMain::execute(ivAppObject *obj, bool useSelectedFrames, bool isFirstPose)
{
    if( !obj->isReady )
        return false;
    
    //if it is the beginning of object motion, keep only the first ivAppObjectFrameData
    if( isFirstPose )
        obj->objData.initCurrentFrameData(true);
    
    ivAppObjAlgorithmList algoList;

    //0. add a new frame data object
    if( !isFirstPose )
        algoList.push_back( new ivAppObjAlgoUseNewFrameData );

    //1. setup
    algoList.push_back( new ivAppObjAlgoSetup );
    
    //2. find nearest frames
    if( !useSelectedFrames )
        algoList.push_back( new ivAppObjAlgoFrontGraphCutFrame );
    else
        algoList.push_back( new ivAppObjAlgoSetFrame );
    
    // check if the selected frame is valid
    algoList.push_back( new ivAppObjAlgoCheckFrame );

    //3. prepare points
    algoList.push_back( new ivAppObjAlgoPreparePoints );
    
    //4. prepare edges
    algoList.push_back( new ivAppObjAlgoPrepareEdges );

    //5. warping object patch
    algoList.push_back( new ivAppObjAlgoSetObjectWarps );
    algoList.push_back( new ivAppObjAlgoCoGEPWarpingObjectPatch );

    algoList.push_back( new ivAppObjAlgoCalcObjectMotionField );

    //6. collage object patch
    algoList.push_back( new ivAppObjAlgoObjectColorGradEdgeTemporalCollage );

    //7. additional shadow constraints
    algoList.push_back( new ivAppObjAlgoAddShadowConstraints );

    //8. warping shadow patch
    algoList.push_back( new ivAppObjAlgoSetShadowWarps );
    algoList.push_back( new ivAppObjAlgoCoGEPWarpingShadowPatch );
    
    algoList.push_back( new ivAppObjAlgoCalcShadowMotionField );
    
    //9. collage shadow patch
    algoList.push_back( new ivAppObjAlgoShadowColorGradEdgeTemporalCollage );
    
    //10. synthesize object patch & depth
    algoList.push_back( new ivAppObjAlgoMakeObject );

    //11. synthesize shadow patch
    algoList.push_back( new ivAppObjAlgoMakeShadow );
    
    //execute the algorithms
    algoList.execute(obj, "Main", true);
    
    //delete
    algoList.release();
    
    return true;
}