//
//  ivAppObjAlgoPreProcess.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjAlgoPreProcess.h"

ivAppObjAlgoPreProcess ivAppObjAlgoPreProcess::inst;

//1. build sim graph
//2. normal estimation
//3. clustering
//4. edge classification
bool ivAppObjAlgoPreProcess::execute(ivAppObject *obj)
{
    if( !obj->isReady )
        return false;
    
    ivAppObjAlgorithmList algoList;
    
    //build object similarity graph
    algoList.push_back( new ivAppObjAlgoBuildSimGraph );
    
    //normal estimation
    algoList.push_back( new ivAppObjAlgoNormalEstimation );
    
    //clustering
    ivAppObjParams &objParams = obj->objParams;
    if( objParams.CLSTR_KM_MS )
        algoList.push_back( new ivAppObjAlgoClustering_KMeans );
    else
        algoList.push_back( new ivAppObjAlgoClustering_MeanShift );
    
    //edge classification
    algoList.push_back( new ivAppObjAlgoEdgeClassification );
    
    //execution
    algoList.execute(obj, "NECEC", true);
    
    //delete
    algoList.release();
    
    return true;
}