//
//  ivAppObjAnimation.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__ivAppObjAnimation__
#define __ivManipulation__ivAppObjAnimation__

#include "ivAppObjPose.h"
#include "ivAppSfM.h"

class ivAppObjAnimation
{
private:
    //parameters
    int intervalNum;
    float controlScl;
    
    //user defined key poses
    std::vector< ivAppObjPose > keyPoses;

    //bezier path control points estimated by keyPoses
    std::vector< ivAppObjPose > controlPoints;
    
    //bezier path calced by controlPoints
    std::vector< ivAppObjPose > vM;
    
    //estimate control points
    //input: keyPoses
    //output: controlPoints
    void estimateBezierControls();
    
    //draw a bezier curve
    //input: starting index of controlPoints
    //output: push_back a path in vM
    void bezierCurveCubic(int key);
    
public:
    //current object pose
    ivAppObjPose objPose;
    
    //reset
    void reset();
    
    //push/pop key poses
    void push();
    void pop();
    
    //goto current object motion
    void gotoMotion(int i);
    Eigen::Matrix4f getMotion(int i);
    int getMotionNum();
    
    //set source object motion
    void setObjMotion(ivAppSfM &objSfM);

    //set bezier path params
    void setAniParams(int intervalNum, float controlScl);
    
    //1. estimateBezierControls
    //2. bezierPathCubic
    //input: keyPoses
    //output: controlPoints
    void estimateObjAnimation();
    
    //render the animation path
    void renderObjAnimationPath();
    
    //read the params
    friend std::istream& operator>>(std::istream& is, ivAppObjAnimation& objani);
    
    //export the params
    friend std::ostream& operator<<(std::ostream& os, const ivAppObjAnimation& objani);
};

#endif /* defined(__ivManipulation__ivAppObjAnimation__) */
