//
//  ivAppObjParams.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjParams.h"

ivAppObjParams::ivAppObjParams()
{
    this->reset();
}

void ivAppObjParams::reset()
{
    this->OSG_oriDiff    = 15.0f;
    this->OSG_frmT       =    60;
    this->NRM_nnum       =    10;
    this->NRM_label      =     1;
    this->CLSTR_KM_MS    = false;
    this->CLSTR_cnum     =     5;
    this->CLSTR_nweight  =  1.0f;
    this->CLSTR_width    = 0.25f;
    this->CLSTR_sharedW  =  0.1f;
    this->CLSTR_edgeDist =  0.2f;
    this->CLSTR_edgeOrnt = 20.0f;
    this->FS_simW        =  0.1f;
    this->WRP_gridSize   =    50;
    this->WRP_rgrid      = false;
    this->WRP_ftrW       =  1.0f;
    this->WRP_simW       =  4.0f;
    this->WRP_edgeW      =  4.0f;
    this->WRP_coW        =  4.0f;
    this->WRP_eintrvl    =    10;
    this->WRP_btmT       =  0.1f;
    this->WRP_btmW       =  4.0f;
    this->CLG_alpha      =  0.0f;
    this->CLG_feature    =100.0f;
    this->CLG_dstrn      = 10.0f;
    this->CLG_sm_edge    = 10.0f;
    this->CLG_sm_clr     =  1.0f;
    this->CLG_sm_grad    =  1.0f;
    this->CLG_tm_fnum    =     2;
    this->CLG_tm_alpha   =  5.0f;
    this->CLG_tm_w       = 20.0f;
    this->CLG_edgeRange  =     2;
    this->CLG_usePoisson = false;
    this->S2D_gridSz     =    30;
    this->S2D_dataW      =  1.0f;
    this->S2D_smthW      =  1.0f;
    this->SHDW_gain      =  1.0f;
    this->SHDW_alpha     =  1.5f;
    this->SHDW_bKSz      =    31;
    this->SHDW_bsigma    = 20.0f;
    this->BZR_scale      =  0.5f;
    this->BZR_interval   =    10;
    this->LT_kd          =  0.1f;
}

//read the params
std::istream& operator>>(std::istream& is, ivAppObjParams& params)
{
    std::string trash;
    is >> trash;
    is >> trash >> trash >> params.OSG_oriDiff;
    is >> trash >> trash >> params.OSG_frmT;
    is >> trash >> trash >> params.NRM_nnum;
    is >> trash >> trash >> params.NRM_label;
    is >> trash >> trash >> params.CLSTR_KM_MS;
    is >> trash >> trash >> params.CLSTR_cnum;
    is >> trash >> trash >> params.CLSTR_nweight;
    is >> trash >> trash >> params.CLSTR_width;
    is >> trash >> trash >> params.CLSTR_sharedW;
    is >> trash >> trash >> params.CLSTR_edgeDist;
    is >> trash >> trash >> params.CLSTR_edgeOrnt;
    is >> trash >> trash >> params.FS_simW;
    is >> trash >> trash >> params.WRP_gridSize;
    is >> trash >> trash >> params.WRP_rgrid;
    is >> trash >> trash >> params.WRP_ftrW;
    is >> trash >> trash >> params.WRP_simW;
    is >> trash >> trash >> params.WRP_edgeW;
    is >> trash >> trash >> params.WRP_coW;
    is >> trash >> trash >> params.WRP_eintrvl;
    is >> trash >> trash >> params.WRP_btmT;
    is >> trash >> trash >> params.WRP_btmW;
    is >> trash >> trash >> params.CLG_alpha;
    is >> trash >> trash >> params.CLG_feature;
    is >> trash >> trash >> params.CLG_dstrn;
    is >> trash >> trash >> params.CLG_sm_edge;
    is >> trash >> trash >> params.CLG_sm_clr;
    is >> trash >> trash >> params.CLG_sm_grad;
    is >> trash >> trash >> params.CLG_tm_fnum;
    is >> trash >> trash >> params.CLG_tm_alpha;
    is >> trash >> trash >> params.CLG_tm_w;
    is >> trash >> trash >> params.CLG_edgeRange;
    is >> trash >> trash >> params.CLG_usePoisson;
    is >> trash >> trash >> params.S2D_gridSz;
    is >> trash >> trash >> params.S2D_dataW;
    is >> trash >> trash >> params.S2D_smthW;
    is >> trash >> trash >> params.SHDW_gain;
    is >> trash >> trash >> params.SHDW_alpha;
    is >> trash >> trash >> params.SHDW_bKSz;
    is >> trash >> trash >> params.SHDW_bsigma;
    is >> trash >> trash >> params.BZR_scale;
    is >> trash >> trash >> params.BZR_interval;
    is >> trash >> trash >> params.LT_kd;
    
    return is;
}

//export the params
std::ostream& operator<<(std::ostream& os, const ivAppObjParams& params)
{
    os << "OBJ_PARAMS" << std::endl;
    os << "\tOSG_oriDiff    : " << params.OSG_oriDiff    << std::endl;
    os << "\tOSG_frmT       : " << params.OSG_frmT       << std::endl;
    os << "\tNRM_nnum       : " << params.NRM_nnum       << std::endl;
    os << "\tNRM_label      : " << params.NRM_label      << std::endl;
    os << "\tCLSTR_KM_MS    : " << params.CLSTR_KM_MS    << std::endl;
    os << "\tCLSTR_cnum     : " << params.CLSTR_cnum     << std::endl;
    os << "\tCLSTR_nweight  : " << params.CLSTR_nweight  << std::endl;
    os << "\tCLSTR_width    : " << params.CLSTR_width    << std::endl;
    os << "\tCLSTR_sharedW  : " << params.CLSTR_sharedW  << std::endl;
    os << "\tCLSTR_edgeDist : " << params.CLSTR_edgeDist << std::endl;
    os << "\tCLSTR_edgeOrnt : " << params.CLSTR_edgeOrnt << std::endl;
    os << "\tFS_simW        : " << params.FS_simW        << std::endl;
    os << "\tWRP_gridSize   : " << params.WRP_gridSize   << std::endl;
    os << "\tWRP_rgrid      : " << params.WRP_rgrid      << std::endl;
    os << "\tWRP_ftrW       : " << params.WRP_ftrW       << std::endl;
    os << "\tWRP_simW       : " << params.WRP_simW       << std::endl;
    os << "\tWRP_edgeW      : " << params.WRP_edgeW      << std::endl;
    os << "\tWRP_coW        : " << params.WRP_coW        << std::endl;
    os << "\tWRP_eintrvl    : " << params.WRP_eintrvl    << std::endl;
    os << "\tWRP_btmT       : " << params.WRP_btmT       << std::endl;
    os << "\tWRP_btmW       : " << params.WRP_btmW       << std::endl;
    os << "\tCLG_alpha      : " << params.CLG_alpha      << std::endl;
    os << "\tCLG_feature    : " << params.CLG_feature    << std::endl;
    os << "\tCLG_dstrn      : " << params.CLG_dstrn      << std::endl;
    os << "\tCLG_sm_edge    : " << params.CLG_sm_edge    << std::endl;
    os << "\tCLG_sm_clr     : " << params.CLG_sm_clr     << std::endl;
    os << "\tCLG_sm_grad    : " << params.CLG_sm_grad    << std::endl;
    os << "\tCLG_tm_fnum    : " << params.CLG_tm_fnum    << std::endl;
    os << "\tCLG_tm_alpha   : " << params.CLG_tm_alpha   << std::endl;
    os << "\tCLG_tm_w       : " << params.CLG_tm_w       << std::endl;
    os << "\tCLG_edgeRange  : " << params.CLG_edgeRange  << std::endl;
    os << "\tCLG_usePoisson : " << params.CLG_usePoisson << std::endl;
    os << "\tS2D_gridSz     : " << params.S2D_gridSz     << std::endl;
    os << "\tS2D_dataW      : " << params.S2D_dataW      << std::endl;
    os << "\tS2D_smthW      : " << params.S2D_smthW      << std::endl;
    os << "\tSHDW_gain      : " << params.SHDW_gain      << std::endl;
    os << "\tSHDW_alpha     : " << params.SHDW_alpha     << std::endl;
    os << "\tSHDW_bKSz      : " << params.SHDW_bKSz      << std::endl;
    os << "\tSHDW_bsigma    : " << params.SHDW_bsigma    << std::endl;
    os << "\tBZR_scale      : " << params.BZR_scale      << std::endl;
    os << "\tBZR_interval   : " << params.BZR_interval   << std::endl;
    os << "\tLT_kd          : " << params.LT_kd          << std::endl;
    return os;
}
