//
//  ivAppObjParams.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__ivAppObjParams__
#define __ivManipulation__ivAppObjParams__

class ivAppObjParams
{
public:
    //init the list of variables
    // OSG_oriDiff  : Orientation Threshold of Object Similarity Graph
    // OSG_frmT     : Frame Threshold of Object Similarity Graph
    // NRM_nnum     : Neighbor Number for Normal Estimation
    // NRM_label    : Uplate Normal by the Labeled Primitives
    // CLSTR_KM_MS  : true->KMeans; false->Meanshift;
    // CLSTR_cnum   : Cluster Number
    // CLSTR_nweight: Normal Weight for Clustering
    // CLSTR_width  : Mean Shift Width
    // CLSTR_sharedW: shared point width
    // CLSTR_edgeDist : Edge Classification tolerant edge distance
    // CLSTR_edgeOrnt : Edge Classification tolerant edge orientation
    // FS_simW      : Frame Selection Graph Cut Similarity Weight
    // WRP_gridSize : Warping Grid Size
    // WRP_rgrid    : Warping Render Grid
    // WRP_ftrW     : Feature Alignment Weight
    // WRP_simW     : Warping Similarity Weight
    // WRP_edgeW    : Warping Edge Weight
    // WRP_coW      : Co-Warping Edge End Point Alignment Weight
    // WRP_eintrvl  : Sampling Rate of Edges
    // WRP_btmT     : Shadow Warping Bottom Threshold
    // WRP_btmW     : Shadow Warping Bottom Weight
    // CLG_alpha    : Weight of Label Color
    // CLG_feature  : Feature weights
    // CLG_dstrn    : Warp Distortion Metric
    // CLG_sm_edge  : Smoothness Weight Edge for Collage
    // CLG_sm_clr   : Smoothness Weight Color for Collage
    // CLG_sm_grad  : Smoothness Weight Gradient for Collage
    // CLG_tm_fnum  : Number of neighboring frames
    // CLG_tm_alpha : Constant of Temporal Coherence
    // CLG_tm_w     : Weight of Temporal Coherence
    // CLG_edgeRange: Edge Range
    // CLG_usePoisson : Apply Poisson
    // S2D_gridSz   : Least Square Depth Grid Width
    // S2D_dataW    : Least Square Depth Data Weight
    // S2D_smthW    : Least Square Depth Smoothness Weight
    // SHDW_gain    : param of shadow brightness
    // SHDW_alpha   : param for to-object distance weight
    // SHDW_bKSz    : bluring kernel size
    // SHDW_bsigma  : bluring kernel gaussian sigma
    // BZR_scale    : Bezier Path Interpolation Parameter
    // BZR_interval : Bezier Path Interval
    // LT_kd        : Diffuse Parameter
    float OSG_oriDiff;
    int   OSG_frmT;
    int   NRM_nnum;
    bool  NRM_label;
    bool  CLSTR_KM_MS;
    int   CLSTR_cnum;
    float CLSTR_nweight;
    float CLSTR_width;
    float CLSTR_sharedW;
    float CLSTR_edgeDist;
    float CLSTR_edgeOrnt;
    float FS_simW;
    int   WRP_gridSize;
    bool  WRP_rgrid;
    float WRP_ftrW;
    float WRP_simW;
    float WRP_edgeW;
    float WRP_coW;
    int   WRP_eintrvl;
    float WRP_btmT;
    float WRP_btmW;
    float CLG_alpha;
    float CLG_feature;
    float CLG_dstrn;
    float CLG_sm_edge;
    float CLG_sm_clr;
    float CLG_sm_grad;
    int   CLG_tm_fnum;
    float CLG_tm_alpha;
    float CLG_tm_w;
    int   CLG_edgeRange;
    bool  CLG_usePoisson;
    int   S2D_gridSz;
    float S2D_dataW;
    float S2D_smthW;
    float SHDW_gain;
    float SHDW_alpha;
    int   SHDW_bKSz;
    float SHDW_bsigma;
    float BZR_scale;
    int   BZR_interval;
    float LT_kd;
    
    ivAppObjParams();
    void reset();
    
    //read the params
    friend std::istream& operator>>(std::istream& is, ivAppObjParams& params);
    
    //export the params
    friend std::ostream& operator<<(std::ostream& os, const ivAppObjParams& params);
};

#endif /* defined(__ivManipulation__ivAppObjParams__) */
