//
//  ivAppObject.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__ivAppObject__
#define __ivManipulation__ivAppObject__

#include "ivDirParams.h"
#include "ivAppScene.h"

#include "ivAppObjParams.h"
#include "ivAppObjAnimation.h"
#include "ivAppObjectVideo.h"
#include "ivAppSfM.h"
#include "ivAppObjectSfMSimGraph.h"
#include "ivAppTvEdges.h"
#include "ivAppGuiParams.h"

#include "ivAppObjectData.h"


class ivAppObject
{
public:
    ivAppObject():isReady(false){}
    ~ivAppObject(){}
    
    bool isReady;

    //object name
    std::string tcName;
    
    //Parameters
    ivAppObjParams objParams;
    
    //user manipulation
    ivAppObjAnimation objAnimation;
    
    //object video
    ivAppObjectVideo objVideo;
    
    //object sfm
    ivAppSfM objSfM;
    
    //transformation for manipulation guidance
    ivAppObjPose mGuidePose;
    float cameraDensity;
    
    //object sfm similarity graph
    ivAppObjectSfMSimGraph objSimGraph;
    
    //user labels
    ivAppTvEdges objTvEdges;
    
    //the data for main algorithm
    ivAppObjectData objData;
    
    //read the object data
    void init( std::string tcName );
    
    //reset status
    void reset();
    
    //render proxy
    void renderProxy
    (bool isActive, bool OBJ_GUIDE, bool OBJ_NCAMERA,
     float objID, bool OBJ_CLSTR, bool OBJ_PROXY, bool SHD_PROXY );
    
    //io
    friend std::istream& operator>>(std::istream& is, ivAppObject& obj);
    friend std::ostream& operator<<(std::ostream& os, const ivAppObject& obj);
};


#endif /* defined(__ivManipulation__ivAppObject__) */
