//
//  ivAppObjectSfMSimGraph.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjectSfMSimGraph.h"

//build similarity graph
void ivAppObjectSfMSimGraph::buildSimGraph
(const ivAppSfM &objSfM, float OSG_oriDiff, int OSG_frmT)
{
    this->OSG_oriDiff = OSG_oriDiff;
    this->OSG_frmT = OSG_frmT;
    
    int fNum = (int)objSfM.vFrameData.size();
    this->simGraphMatrix.resize(fNum, fNum);
    this->simGraphMatrix.setConstant( 0.0f );
    
    //for each i!=j, i<j
    for(int i=0;i<fNum;i++)for(int j=i+1;j<fNum;j++)
    {
        const Eigen::Vector3f &uCami = objSfM.vFrameData[i].unitCam;
        const Eigen::Vector3f &uCamj = objSfM.vFrameData[j].unitCam;
        
        //calc orientation difference in degrees
        float ornDiff = 1000.0f;
        if( j-i<=OSG_frmT )
        {
            float dotValue = uCami.dot(uCamj);
            dotValue = std::max( dotValue, -1.0f );
            dotValue = std::min( dotValue, 1.0f );
            ornDiff = std::acos( uCami.dot(uCamj) ) * 180.0f / M_PI;
        }
        
        this->simGraphMatrix(i,j) = ornDiff;
        this->simGraphMatrix(j,i) = ornDiff;
    }
}

//if i, j are neighbor
bool ivAppObjectSfMSimGraph::isNeighbor(int i, int j)const
{
    return this->simGraphMatrix(i,j) < this->OSG_oriDiff;
}

//get the error from i to j
float ivAppObjectSfMSimGraph::operator()(int i, int j)const
{
    return this->simGraphMatrix(i,j);
}
