//
//  ivAppObjectVideo.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppObjectVideo.h"


//initialization
void ivAppObjectVideo::init(std::string tcName)
{
    //read object video
    std::string videoPath = ivDirParams::getVideoPath(tcName);
    this->objVideo.readVideo( videoPath, INT_MAX );
    this->objVideo.calcMeanLuminance();
    
    //read object mask
    std::string maskPath = ivDirParams::getMaskPath(tcName);
    this->objMask.readVideo( maskPath, INT_MAX );
    
    //initialization
    Utils_Ocv &uocv = Utils_Ocv::instance();
    int frameNum = objMask.frameNum;
    this->objROIs.resize( frameNum );
    this->objCntrMasks.resize( frameNum );
    for(int i=0;i<frameNum;i++)
    {
        cv::Mat &curMask = objMask.frames[i];
        cv::cvtColor( curMask, curMask, CV_RGB2GRAY, 1);
    
        //append alpha channel to the object
        cv::Mat &curFrm = objVideo.frames[i];
        curFrm = uocv.appendAlpha( curFrm, curMask );
        
        //FG ROI
        cv::Rect &roi = objROIs[i];
        uocv.getFGBB( roi, curMask );
        
        //keep only broi
        this->getFGRegion(curFrm, i);
        this->getFGRegion(curMask, i);
        
        //calc contour map
        int contourWidth = ( roi.width + roi.height ) / 2 / 5;
        this->objCntrMasks[i] = Utils_Ocv::instance().getContourMap( curMask , contourWidth );
    }
    
    this->isReady = true;
}

//crop object region
void ivAppObjectVideo::getFGRegion(cv::Mat &img, int nframeID) const
{
    //FG ROI
    cv::Rect roi = objROIs[nframeID];
    img = img(roi).clone();
    cv::copyMakeBorder( img, img, BORDER_SZ, BORDER_SZ, BORDER_SZ, BORDER_SZ, cv::BORDER_CONSTANT, cv::Scalar::all(0) );
    
    //make the width 4 multiple
    int wOffset = ( 4 - img.cols%4 ) % 4;
    cv::copyMakeBorder( img, img, 0, 0, 0, wOffset, cv::BORDER_CONSTANT, cv::Scalar::all(0) );
}

//map point to roi space
bool ivAppObjectVideo::remapPoint(cv::Point2f &p, int nframeID) const
{
    const cv::Rect &roi = this->objROIs[nframeID];
    p.x = p.x - roi.x + BORDER_SZ;
    p.y = p.y - roi.y + BORDER_SZ;
    
    cv::Size sz = this->objVideo.frames[nframeID].size();
    return p.x>=0 && p.y>=0 && p.x<sz.width && p.y<sz.height;
}

bool ivAppObjectVideo::isNearContour(const cv::Point2f &p, int nframeID) const
{
    return this->objCntrMasks[nframeID].at<uchar>(p.y, p.x) > 128;
}

