//
//  ivAppObjects.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__ivAppObjects__
#define __ivManipulation__ivAppObjects__

#import "IndexedCircularList.hpp"
#import "ivAppObject.h"
#import "ivAppObjAlgoPreProcess.h"
#import "ivAppObjAlgoCBFNF.h"
#import "ivAppObjAlgoSelectFrames.h"
#import "ivAppObjAlgoMain.h"

class ivAppObjects : public IndexedCircularList<ivAppObject>
{
private:
    ivAppObjects(){}
    ~ivAppObjects(){}
    ivAppObjects(const ivAppObjects &);
    ivAppObjects& operator=(const ivAppObjects&);
    static ivAppObjects inst;
    
    //preprocess object
    void goPreProcess();

public:
    static ivAppObjects& instance(){return inst;}
    
    //cull back and find nearest frame for each object
    void goCBFNFs(bool useSelectedFrames);
    
    //go selecting frames
    void goSelectFrames();
    
    //go main for each object
    void goMains(bool useSelectedFrames, bool isFirstPose);
    
    //get the max animation length
    int getAniMax();
    
    //set object motion
    void setMotion(int i);
    
    //set parameters for exporting
    void setCLG_alpha(float alpha);
    
    //rendering
    void renderObjects(bool OBJ_EDGE, BOOL DEPTHMAP);
    void renderShadows(bool OBJ_EDGE);
    void renderProxies(bool OBJ_GUIDE, bool OBJ_NCAMERA,
                       bool OBJ_CLSTR, bool OBJ_PROXY, bool SHD_PROXY);
    void renderObectMotions();
    
    //io
    friend std::istream& operator>>(std::istream& is, ivAppObjects &objects);
    friend std::ostream& operator<<(std::ostream& os, const ivAppObjects &objects);
};

#endif /* defined(__ivManipulation__ivAppObjects__) */
