//
//  ivRenderOrder.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivRenderOrder.h"

ivRenderOrder ivRenderOrder::inst;

//calculate Rendering Order Table
void ivRenderOrder::calcROT()
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    int animationType = guiParams.aniType;
    
    ivAppScene &scene = ivAppScene::instance();
    ivAppObjects &objects = ivAppObjects::instance();
    
    if( !scene.isReady )
    {
        guiParams.scnPos    = 0;
        guiParams.scnMaxPos = 0;
        guiParams.objPos    = 0;
        guiParams.objMaxPos = 0;
        guiParams.aniPos    = 0;
        guiParams.aniMaxPos = 0;
        return;
    }
    
    int frameMaxPos = scene.scnVideo.frameNum - 1;
    int objMaxMotionPos = objects.getAniMax();

    ROT.clear();

    //Static Objects & Dynamic Scene
    if( animationType == 0 )
    {
        for(int i=0;i<=frameMaxPos;i++)
            ROT.push_back( scnobjPose(i,-1) );
    }
    //Dynamic Objects & Static Scene
    else if( animationType == 1 )
    {
        for(int i=0;i<=objMaxMotionPos;i++)
            ROT.push_back( scnobjPose(-1,i) );
    }
    //Dynamic Objects & Dynamic Scene
    else if( animationType == 2 )
    {
        if( objMaxMotionPos >= frameMaxPos )
        for(int i=0;i<=objMaxMotionPos;i++)
        {
            int j = (int)((float)i*frameMaxPos/objMaxMotionPos);
            ROT.push_back( scnobjPose(j, i) );
        }
        else
        for(int i=0;i<=frameMaxPos;i++)
        {
            int j = (int)((float)i*objMaxMotionPos/frameMaxPos);
            ROT.push_back( scnobjPose(i, j) );
        }
    }
    
    guiParams.scnMaxPos = frameMaxPos;
    if( guiParams.scnPos > guiParams.scnMaxPos )
        guiParams.scnPos = guiParams.scnMaxPos;
    
    if( !objects.empty() )
    {
        ivAppObject *obj = objects.getCurrentObject();
        int motionNum = obj->objAnimation.getMotionNum();
        if( motionNum == 0 )
            guiParams.objMaxPos = guiParams.objPos = 0;
        else
        {
            guiParams.objMaxPos = obj->objAnimation.getMotionNum()-1;
            if( guiParams.objPos > guiParams.objMaxPos )
                guiParams.objPos = guiParams.objMaxPos;
        }
    }
    
    guiParams.aniMaxPos = this->getMaxPos();
    if( guiParams.aniPos > guiParams.aniMaxPos )
        guiParams.aniPos = guiParams.aniMaxPos;
    if( guiParams.aniPos < 0 )
        guiParams.aniPos = 0;
}

//set animation pose
void ivRenderOrder::setAnimationPose()const
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    int aniPose = guiParams.aniPos;
    
    if( aniPose<0 || aniPose>=(int)this->ROT.size() )
        return;
    
    const scnobjPose &soPose = this->ROT[aniPose];
    
    if( soPose.scenePose >= 0 )
        guiParams.scnPos = soPose.scenePose;
    
    if( soPose.objPose >= 0 )
    {
        ivAppObjects &objects = ivAppObjects::instance();
        guiParams.objPos = soPose.objPose;
        objects.setMotion( soPose.objPose );
    }
}

int ivRenderOrder::getMaxPos()
{
    return (int)this->ROT.size() - 1;
}