//
//  ivAppScene.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppScene.h"

ivAppScene ivAppScene::inst;

//initialization
void ivAppScene::init( std::string tcName )
{
    this->tcName = tcName;
    
    std::string videoPath = ivDirParams::getVideoPath(tcName);
    scnVideo.readVideo( videoPath, INT_MAX );
    scnVideo.calcMeanLuminance();
    
    std::string objsfmPath = ivDirParams::getObjSfMPath(tcName);
    scnSfM.readObjSfM(objsfmPath);
    
    //static scene & moving object
    if( tcName[0]=='M' )
    {
        int fNum = (int)this->scnSfM.vFrameData.size();
        for(int i=1;i<fNum;i++)
            this->scnSfM.vFrameData[i] = this->scnSfM.vFrameData[0];
    }
    
    //init scene light
    scnLight.pos = this->scnSfM.vFrameData[0].camPosition;
    
    this->isReady = true;
}

//estimate ground depth
void ivAppScene::estimateGroundDepth()
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    const Eigen::Matrix4f &KM = scnSfM.vFrameData[guiParams.scnPos].KM;

    Plane subGround(0.0f,0.0f,1.0f,0.0f);
    this->groundDepthMap = subGround.getDepthMap( KM, cv::Size(OSZ_WIDTH, OSZ_HEIGHT) );
}

//io
std::istream& operator>>(std::istream& is, ivAppScene& scene)
{
    std::string trash;
    is >> trash;
    
    //scene video
    is >> scene.tcName;
    scene.init(scene.tcName);

    //scene light
    is >> scene.scnLight;

    is >> trash;
    
    return is;
}

std::ostream& operator<<(std::ostream& os, const ivAppScene& scene)
{
    os << "====================SCENE====================" << std::endl;
    //scene video
    os << scene.tcName << std::endl;

    //scene light
    os << scene.scnLight;
    os << "==================EOF_SCENE==================" << std::endl;
    return os;
}