//
//  ivAppGuiParams.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivAppGuiParams.h"

@implementation ivAppGuiParams

static ivAppGuiParams *inst;
+ (ivAppGuiParams*) instance
{
    return inst;
}

//manipulation mode object/light
@synthesize mMode;

//animation pose
@synthesize scnPos;
@synthesize scnMaxPos;
@synthesize objPos;
@synthesize objMaxPos;
@synthesize aniPos;
@synthesize aniMaxPos;
@synthesize aniType;

//input video list
@synthesize sceneList;
@synthesize objectList;

//the selected test case
@synthesize selectedScene;
@synthesize selectedObject;

- (id) init
{
    @synchronized(self)
    {
        if ( inst == nil )
        {
            inst = [super init];
            
            //start initialization
            NSOpenPanel *openPanel = [NSOpenPanel openPanel];
            openPanel.title = @"Open The Working Folder.";
            [openPanel setCanChooseDirectories:YES];
            [openPanel setCanChooseFiles:NO];
            NSInteger status = [openPanel runModal];
            if( status != NSModalResponseOK )
                exit(EXIT_SUCCESS);

            std::string wPath = [[[[openPanel URL] path] stringByAppendingString:@"/"] UTF8String];
            chdir(wPath.c_str());
            
            //init the list of the input video
            sceneList  = [[NSMutableArray alloc] init];
            objectList = [[NSMutableArray alloc] init];
            std::vector<std::string> dirs = Utils_DirList::parseDirList( ivDirParams::videoDir );
            std::vector<std::string>::iterator it = dirs.begin();
            bool findFirstObject = false;
            for(;it!=dirs.end();it++)
            {
                if( it->substr(0,3) != "SCN" )
                {
                    [objectList addObject:@(it->c_str())];
                    
                    if( !findFirstObject )
                    {
                        findFirstObject = true;
                        selectedObject = @(it->c_str());
                    }
                }

                [sceneList addObject:@(it->c_str())];
            }
            
            selectedScene  = @(dirs[0].c_str());
            
            [self reset];
        }
    }
    
    return inst;
}

-(void)reset
{
    scnPos = scnMaxPos = 0;
    objPos = objMaxPos = 0;
    aniPos = aniMaxPos = 0;
    aniType = 1;
}

#pragma mark Rendering States
-(void)setRState_Manipulation
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"1", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"1", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map

     @"1", @"OBJ_PROXY",  //object proxy
     @"0", @"OBJ_PATCH",  //object patch
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"1", @"SHD_PROXY",  //shadow proxy
     @"0", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}
-(void)setRState_Structure
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"0", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"1", @"SCN_GROUND", //scene ground
     @"0", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"0", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map
     
     @"1", @"OBJ_PROXY",  //object proxy
     @"0", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"0", @"SHD_PROXY",  //shadow proxy
     @"0", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}
-(void)setRState_Results
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"0", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"0", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map

     @"0", @"OBJ_PROXY",  //object proxy
     @"1", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"0", @"SHD_PROXY",  //shadow proxy
     @"1", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}

-(void)setRState_ObjectPreview
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"1", @"SCN_LIGHT",  //scene light

     @"0", @"OBJ_CLSTR",  //object clusters
     @"1", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map
     
     @"1", @"OBJ_PROXY",  //object proxy
     @"0", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"1", @"SHD_PROXY",  //shadow proxy
     @"0", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}
-(void)setRState_ObjectCollageEdge
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"0", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"0", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map
     
     @"0", @"OBJ_PROXY",  //object proxy
     @"1", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"0", @"SHD_PROXY",  //shadow proxy
     @"1", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}
-(void)setRState_ObjectCollageFeature
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"0", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"0", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map
     
     @"1", @"OBJ_PROXY",  //object proxy
     @"1", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"0", @"SHD_PROXY",  //shadow proxy
     @"1", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}
-(void)setRState_ObjectCollage
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams->rStates =
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
     @"1", @"SCN_VIDEO",  //scene video
     @"0", @"SCN_SfM",    //scene sfm
     @"0", @"SCN_GROUND", //scene ground
     @"0", @"SCN_LIGHT",  //scene light
     
     @"0", @"OBJ_CLSTR",  //object clusters
     @"0", @"OBJ_MOTION", //animation path
     @"0", @"OBJ_GUIDE",  //object manipulation guidance
     @"0", @"OBJ_NCAMERA",//object nearest cameras
     @"0", @"DEPTHMAP",   //depth map
     
     @"0", @"OBJ_PROXY",  //object proxy
     @"1", @"OBJ_PATCH",  //object collage
     @"0", @"OBJ_EDGE",   //object edge collage
     
     @"0", @"SHD_PROXY",  //shadow proxy
     @"1", @"SHD_PATCH",  //shadow patch
     @"0", @"SHD_EDGE",   //shadow edge collage
     
     nil];
    
    [self.tableview_rState reloadData];
}

@end
