//
//  ivAppGodView.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivAppGodView.h"

ivAppGodView ivAppGodView::inst;

//reset parameters
void ivAppGodView::reset()
{
    //initialization
    targetScale = 2.0f;
    alpha = 0.0f;
    beta = M_PI / 3.0f;
    
    srcAxis = true;
    motionAxis = true;
}

//sub rendering functions
void ivAppGodView::drawObjectData()
{
    ivAppObjects &objects = ivAppObjects::instance();
    ivAppObject *obj = objects.getCurrentObject();
    
    Utils_Ogl::drawAxis();
    obj->objSfM.renderPoints( 1.0f );
    obj->objSfM.renderCameraPath( 2.0f, 0.5f, false );
    
    if( !srcAxis )
        return;
    
    const int fNum = (int)obj->objSfM.vFrameData.size();
    
    //draw end cameras
    glPushAttrib(GL_CURRENT_BIT);
    glColor4ub(0,0,0,255);
    obj->objSfM.renderCameraCube( 0.2f,      0, false );
    obj->objSfM.renderCameraCube( 0.2f, fNum-1, false );
    glPopAttrib();
    
    //draws cameras
    for(int i=0;i<fNum;i++)
    {
        if( i%(int)obj->cameraDensity!=0 && i<fNum-1 )
            continue;
        else
            obj->objSfM.renderCameraPyramid( 0.15f, i, false );
    }
    
}
void ivAppGodView::drawAnimationData()
{
    ivAppScene &scene = ivAppScene::instance();
    ivAppObjects &objects = ivAppObjects::instance();
    ivAppObject *obj = objects.getCurrentObject();
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivRenderOrder &renderOrder = ivRenderOrder::instance();
    
    int motionNum = renderOrder.getMaxPos()+1;
    std::vector<Eigen::Vector4f> lights(motionNum);
    std::vector<Eigen::Matrix4f> gInvScnMInvs(motionNum);
    for(int i=0;i<motionNum;i++)
    {
        int scnPos = renderOrder.ROT[i].scenePose;
        if( scnPos < 0 )
            scnPos = guiParams.scnPos;
        
        int objPos = renderOrder.ROT[i].objPose;
        if( objPos < 0 )
            objPos = guiParams.objPos;
        
        const Eigen::Matrix4f &scnInvM = scene.scnSfM.vFrameData[scnPos].inverseM;
        Eigen::Matrix4f objInvM = obj->objAnimation.getMotion(objPos).inverse();
        gInvScnMInvs[i] = objInvM * scnInvM;
        lights[i] = objInvM * scene.scnLight.pos;
    }
    
    glPushAttrib( GL_LINE_BIT | GL_POINT_BIT );
    glPointSize(1.0f);
    glLineWidth(1.0f);
    
    //render object animation orientation
    if( motionAxis )
        for(int i=0;i<motionNum;i++)
        {
            glPushMatrix();
            glMultMatrixf( gInvScnMInvs[i].data() );
            glScalef(0.5f,0.5f,0.5f);
            Utils_Ogl::drawAxis();
            glPopMatrix();
        }
    
    //render object animation path
    glColor4ub(255,255,255,255);
    glBegin(GL_LINE_STRIP);
    for(int i=0;i<motionNum;i++)
        glVertex4fv( gInvScnMInvs[i].col(3).data() );
    glEnd();
    
    //render light path
    glColor4ub(255,255,0,255);
    glBegin(GL_LINE_STRIP);
    for(int i=0;i<motionNum;i++)
        glVertex4fv( lights[i].data() );
    glEnd();
    
    glPopAttrib();
}
void ivAppGodView::drawSelectedFrameCameras()
{
    ivAppObjects &objects = ivAppObjects::instance();
    const ivAppObject *obj = objects.getCurrentObject();
    const ivAppSfM &objSfM = obj->objSfM;
    const ivAppObjectData &objData = obj->objData;
    const std::vector<ivAppObjectPart> &objParts = objData.parts;
    const ivAppObjectFrameData &objFrameData = objData.frames.back();
    
    const int clusterNum = (int)objParts.size();
    for(int i=0;i<clusterNum;i++)
    {
        const int aniPosNum = (int)objData.objNFrames.cols();
        
        //selected frames for object motion
        for(int j=0;j<aniPosNum;j++)
        {
            glColor4ub( 255, 255, 255, 255 );
            if( !objData.ifObjectMotionFramesValid[j] )
                glColor4ub( 192, 0, 0, 255 );
            
            int nframeID = objData.objNFrames(i,j).nframeID;
            objSfM.renderCameraSphere( 0.1, nframeID, false );
        }
        
        //draw current frame for object using box
        {
            glColor4ub( 255, 255, 255, 255 );
            if( !objFrameData.ifObjectPoseFramesValid )
                glColor4ub( 192, 0, 0, 255 );
    
            int nframeID = objFrameData[i].object.nfData.nframeID;
            objSfM.renderCameraPyramid( 0.3f, nframeID, false );
            objSfM.renderCameraSphere( 0.1, nframeID, false );
        }
    
        //selected frames for shadow motion
        for(int j=0;j<aniPosNum;j++)
        {
            glColor4ub( 255, 255, 0, 255 );
            if( !objData.ifShadowMotionFramesValid[j] )
                glColor4ub( 192, 0, 0, 255 );
        
            int nframeID = objData.shdNFrames(i,j).nframeID;
            objSfM.renderCameraSphere( 0.08, nframeID, false );
        }

        //draw current frame for shadow using box
        {
            glColor4ub( 255, 255, 0, 255 );
            if( !objFrameData.ifShadowPoseFramesValid )
                glColor4ub( 192, 0, 0, 255 );
            
            int nframeID = objFrameData[i].shadow.nfData.nframeID;
            objSfM.renderCameraPyramid( 0.24f, nframeID, false );
            objSfM.renderCameraSphere( 0.08, nframeID, false );
        }
    }
}
void ivAppGodView::drawCurrentState()
{
    const ivAppScene &scene = ivAppScene::instance();
    const ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppObjects &objects = ivAppObjects::instance();
    ivAppObject *obj = objects.getCurrentObject();
    
    Eigen::Matrix4f gInv = obj->objAnimation.objPose.gTransform().inverse();
    const Eigen::Matrix4f &inverseM = scene.scnSfM.vFrameData[guiParams.scnPos].inverseM;
    
    //render current scene camera
    glPushAttrib(GL_LINE_BIT);
    glPushMatrix();
    {
        glLineWidth(3.0);
        Eigen::Matrix4f camMatrixInv = gInv * inverseM;
        glMultMatrixf( camMatrixInv.data() );
        Utils_Ogl::drawAxis();
    }
    glPopMatrix();
    glPopAttrib();

    //render the scene light
    glPushMatrix();
    {
        glColor4ub(255,255,0,255);
        Eigen::Vector4f lightPos = gInv * scene.scnLight.pos;
        glTranslatef( lightPos(0), lightPos(1), lightPos(2) );
        glScalef( 0.15f, 0.15f, 0.15f );
        Utils_Ogl::drawSphere(10,10);
    }
    glPopMatrix();
}

void ivAppGodView::drawGodView()
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    Utils_Ogl::drawBackgroundColor(0.7882f, .8156f, .8431f);
    
    ivAppScene &scene = ivAppScene::instance();
    ivAppObjects &objects = ivAppObjects::instance();
    if( !scene.isReady || objects.empty() )
        return;
    
    ivAppObject *curObj = objects.getCurrentObject();
    if( !curObj->isReady )
        return;
    
    glPushAttrib( GL_TRANSFORM_BIT );
    //=======================================================================
    
    glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    float asp = (float)OSZ_WIDTH/OSZ_HEIGHT;
    glOrtho( -asp, asp, -1.0, 1.0, 1.0, -1.0);
    glFrustum( -0.2, 0.2, -0.2, 0.2, 0.2, 10.0 );
    
    float smRadius = std::cos(beta);
    GLdouble eyeX = smRadius * std::cos(alpha);
    GLdouble eyeY = smRadius * std::sin(alpha);
    GLdouble eyeZ = std::sin(beta);;
    gluLookAt( eyeX, eyeY, eyeZ, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 );
    
//==
    glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();
    glScalef(targetScale, targetScale, targetScale);
    
    //render the plane
    glColor4ub(128,128,128,255);
    Utils_Ogl::whiteGridXY(10,10);
    
    glScalef(.1f, .1f, .1f);
    
    //drawing functions
    drawObjectData();
    drawAnimationData();
    drawSelectedFrameCameras();
    drawCurrentState();
    
    glPopMatrix();
//==
    
    glMatrixMode( GL_PROJECTION );
    glPopMatrix();

    //=======================================================================
    glPopAttrib();
}

//io
std::istream& operator>>(std::istream& is, ivAppGodView& godView)
{
    std::string trash;
    
    is >> trash;
    is >> trash >> trash >> godView.targetScale;
    is >> trash >> trash >> godView.alpha;
    is >> trash >> trash >> godView.beta;
    
    return is;
}

std::ostream& operator<<(std::ostream& os, const ivAppGodView& godView)
{
    os << "GODVIEW_PARAMS" << std::endl;
    os << "\ttargetScale    : " << godView.targetScale << std::endl;
    os << "\talpha          : " << godView.alpha       << std::endl;
    os << "\tbeta           : " << godView.beta        << std::endl;
    return os;
}






