//
//  ivAppRenderer.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppRenderer.h"

ivAppRenderer ivAppRenderer::inst;

void ivAppRenderer::render()
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    Utils_Ogl::drawBackgroundColor(0.7882f, .8156f, .8431f);
    
    ivAppScene &scene = ivAppScene::instance();
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppObjects &objects = ivAppObjects::instance();
    
    //render switches
    bool SCN_VIDEO   = [[guiParams->rStates objectForKey:@"SCN_VIDEO"  ] boolValue];
    bool SCN_SfM     = [[guiParams->rStates objectForKey:@"SCN_SfM"    ] boolValue];
    bool SCN_GROUND  = [[guiParams->rStates objectForKey:@"SCN_GROUND" ] boolValue];
    bool SCN_LIGHT   = [[guiParams->rStates objectForKey:@"SCN_LIGHT"  ] boolValue];

    bool OBJ_CLSTR   = [[guiParams->rStates objectForKey:@"OBJ_CLSTR"  ] boolValue];
    bool OBJ_MOTION  = [[guiParams->rStates objectForKey:@"OBJ_MOTION" ] boolValue];
    bool OBJ_GUIDE   = [[guiParams->rStates objectForKey:@"OBJ_GUIDE"  ] boolValue];
    bool OBJ_NCAMERA = [[guiParams->rStates objectForKey:@"OBJ_NCAMERA"] boolValue];
    bool DEPTHMAP    = [[guiParams->rStates objectForKey:@"DEPTHMAP"   ] boolValue];
    
    bool OBJ_PROXY   = [[guiParams->rStates objectForKey:@"OBJ_PROXY"  ] boolValue];
    bool OBJ_PATCH   = [[guiParams->rStates objectForKey:@"OBJ_PATCH"  ] boolValue];
    bool OBJ_EDGE    = [[guiParams->rStates objectForKey:@"OBJ_EDGE"   ] boolValue];
    
    bool SHD_PROXY   = [[guiParams->rStates objectForKey:@"SHD_PROXY"  ] boolValue];
    bool SHD_PATCH   = [[guiParams->rStates objectForKey:@"SHD_PATCH"  ] boolValue];
    bool SHD_EDGE    = [[guiParams->rStates objectForKey:@"SHD_EDGE"   ] boolValue];
    
    if( !scene.isReady )
        return;
    
    //cull back and find nearest frame
    objects.goCBFNFs(false);
    
    frameData &fd_Scn = scene.scnSfM.vFrameData[guiParams.scnPos];
    Utils_Ogl::setMatrix( scene.scnSfM.width, scene.scnSfM.height, scene.scnSfM.K, fd_Scn.M );
    
    //=====================================================================
    // Scene
    //=====================================================================
    if( SCN_VIDEO )
    {
        glDisable(GL_DEPTH_TEST);
        Utils_Ogl::setMatrixIdentity();
        Utils_Ogl::renderMat( scene.scnVideo.frames[guiParams.scnPos] );
        Utils_Ogl::resetMatrix();
        glEnable(GL_DEPTH_TEST);
    }
    
    if( SCN_GROUND )
    {
        glPushAttrib( GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT );
        glDisable(GL_DEPTH_TEST);
        glColor4ub(0,0,0,255);
        
        glPushMatrix();
        glScalef(5.0f,5.0f,5.0f);
        Utils_Ogl::whiteGridXY(25,25);
        glPopMatrix();
        
        glPopAttrib();
    }
    
    //=====================================================================
    // Object Patches
    //=====================================================================
    glDisable(GL_DEPTH_TEST);
    Utils_Ogl::setMatrixIdentity();

    if( SHD_PATCH )
        objects.renderShadows(SHD_EDGE);
    
    if( OBJ_PATCH || DEPTHMAP )
        objects.renderObjects(OBJ_EDGE, DEPTHMAP);
    
    Utils_Ogl::resetMatrix();
    glEnable(GL_DEPTH_TEST);
    
    //=====================================================================
    // Object Structure
    //=====================================================================

    if( SCN_SfM )
        ((ivAppSfM)(scene.scnSfM)).render(guiParams.scnPos);
    
    if( SCN_LIGHT )
        scene.scnLight.render();
    
    //render object shadow proxy
    objects.renderProxies(OBJ_GUIDE, OBJ_NCAMERA,
                          OBJ_CLSTR, OBJ_PROXY, SHD_PROXY);
    
    if( OBJ_MOTION )
        objects.renderObectMotions();
 
    Utils_Ogl::resetMatrix();
}