//
//  ivAppDriver.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivAppDriver.h"

@implementation ivAppDriver

static ivAppDriver *inst;
+ (ivAppDriver*) instance
{
    return inst;
}
- (id) init
{
    @synchronized(self)
    {
        if ( inst == nil )
            inst = [super init];
    }
    
    return inst;
}


-(void)awakeFromNib
{
    [self setRState_Manipulation];
}

#pragma mark - View Call Back

//setup animation
-(void)setupAnimation
{
    //cal rendering order table
    ivRenderOrder &ivRO = ivRenderOrder::instance();
    ivRO.calcROT();
    
    [self setRState_Manipulation];
}

-(void)setObjPose
{
    ivAppObjects &objects = ivAppObjects::instance();
    if( !objects.empty() )
    {
        ivAppObject *obj = objects.getCurrentObject();
        ivAppGuiParams *guiParams = [ivAppGuiParams instance];
        obj->objAnimation.gotoMotion( guiParams.objPos );
    }
}

-(void)setAniPose
{
    const ivRenderOrder &ivRO = ivRenderOrder::instance();
    ivRO.setAnimationPose();
    [self setRState_Manipulation];
}

//switch to the scene
-(void)gotoScene
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    std::string sceneCase = [guiParams.selectedScene UTF8String];
    
    ivAppScene &scene = ivAppScene::instance();
    scene.init( sceneCase );
    
    guiParams.scnPos = 0;
    [self setupAnimation];
    [self setRState_Manipulation];
}

//modify object list
-(void)addObject
{
    ivAppScene &scene = ivAppScene::instance();
    if( !scene.isReady )
        return;
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    std::string objCase = [guiParams.selectedObject UTF8String];
    
    ivAppObject obj;
    ivAppObjects &objects = ivAppObjects::instance();
    objects.addObject(obj);
    ivAppObject *curObj = objects.getCurrentObject();
    
    curObj->init( objCase );
    [self preProcessObj];
    
    guiParams.objPos = 0;
    [self setObjPose];
    [self setupAnimation];
    [self.tableview_Patameters reloadData];
    [self setRState_Manipulation];
}
-(void)rmCurObject
{
    ivAppObjects &objects = ivAppObjects::instance();
    objects.rmCurObject();
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams.objPos = 0;
    [self setObjPose];
    [self setupAnimation];
    [self.tableview_Patameters reloadData];
    [self setupAnimation];
    [self setRState_Manipulation];
}
-(void)duplicateObject
{
    ivAppObjects &objects = ivAppObjects::instance();
    objects.duplicateObject();
    
    if( !objects.empty() )
    {
        ivAppObject *obj = objects.getCurrentObject();
        obj->objAnimation.reset();
    }
    
    [self.tableview_Patameters reloadData];
    [self setRState_Manipulation];
}
-(void)nextObject
{
    ivAppObjects &objects = ivAppObjects::instance();
    objects.nextObject();
    
    [self.tableview_Patameters reloadData];
    [self setRState_Manipulation];
}
-(void)prevObject
{
    ivAppObjects &objects = ivAppObjects::instance();
    objects.prevObject();
    
    [self.tableview_Patameters reloadData];
    [self setRState_Manipulation];
}

//rest the object
-(void)resetObject
{
    ivAppObjects &objects = ivAppObjects::instance();
    if( objects.empty() )
        return;
    
    ivAppObject *curObject = objects.getCurrentObject();
    curObject->objAnimation.reset();
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams.objPos = 0;
    [self setObjPose];
    [self setupAnimation];
    [self setRState_Manipulation];
}

//preprocess object
-(void)preProcessObj
{
    ivAppObjects &objects = ivAppObjects::instance();
    if( objects.empty() )
        return;
    
    ivAppObject *curObject = objects.getCurrentObject();
    ivAppObjAlgoPreProcess::instance().execute( curObject );
    
    [self setRState_Manipulation];
}

//select frames
-(void)selectFrames
{
    ivAppObjects &objects = ivAppObjects::instance();
    if( objects.empty() )
        return;
    ivAppObject *curObj = objects.getCurrentObject();
    ivAppObjAlgoSelectFrames::instance().execute(curObj);
}

//process
-(void)process
{
    ivAppObjects &objects = ivAppObjects::instance();
    objects.goMains( false, true );
    
    [self setRState_Results];
}

#pragma mark - Menu Call Back

// Render : set rendering mode
-(void)setRState_Manipulation
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    [guiParams setRState_Manipulation];
}
-(void)setRState_Structure
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    [guiParams setRState_Structure];
}
-(void)setRState_Results
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    [guiParams setRState_Results];
}

#pragma mark - Exporting

-(void)captureImage:(std::string)imgPath chnNum:(int)cn
{
    Utils_Ogl::FBO fbObject;
    fbObject.start( OSZ_WIDTH, OSZ_HEIGHT );
    
    ivAppRenderer &renderer = ivAppRenderer::instance();
    renderer.render();
    
    fbObject.writeFrameBuffer( imgPath, cn );

    fbObject.end();
}

-(void)exportVideo:(std::string)videoPath FPS:(double)fps
{
    // Redirect cout.
    std::streambuf* oldCoutStreamBuf = std::cout.rdbuf();
    std::ostringstream strCout;
    std::cout.rdbuf( strCout.rdbuf() );
    
    ivAppObjects &objects = ivAppObjects::instance();
    objects.goSelectFrames();
    
    Utils_Ogl::FBO fbObject;
    fbObject.start(OSZ_WIDTH,OSZ_HEIGHT);
    cv::Size osz(OSZ_WIDTH, OSZ_HEIGHT);
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppRenderer &renderer = ivAppRenderer::instance();
    
    VideoOutput voObjectCollage( videoPath, osz, fps );
    
    Utils_Timer jsTimer;
    jsTimer.start("ExportVideo");
    
    //Rendering
    for(int i=0;i<=guiParams.aniMaxPos;i++)
    {
        std::cout << "output frame " << i << "/" << guiParams.aniMaxPos << std::endl;
        
        guiParams.aniPos = i;
        [self setAniPose];
        
        objects.goMains( true, i==0 );
        [self setRState_Results];
        
        //Object Collage
        [guiParams setRState_ObjectCollage];
        renderer.render();
        cv::Mat objectCollageMap = fbObject.readFrameBuffer(3);
        voObjectCollage.writeFrame( objectCollageMap );
        glSwapAPPLE();
    }
    
    voObjectCollage.finishWriting();

    jsTimer.stop("ExportVideo");
    
    // Restore old cout.
    std::cout.rdbuf( oldCoutStreamBuf );
    
    std::string logFile = videoPath.substr(0,videoPath.length()-4) + "_log.txt";
    std::ofstream ofs(logFile);
    ofs << strCout.str();
    ofs.close();
    
    fbObject.end();
}

#pragma mark - Mouse Call Back

//drag the mouse
- (void)dragCallBack:(NSPoint)tgtPoint
{
    ivAppScene &scene = ivAppScene::instance();
    
    if( !scene.scnVideo.isReady )
        return;
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppObjects &objects = ivAppObjects::instance();
    
    Eigen::Vector2f pt2d;
    pt2d << tgtPoint.x, scene.scnVideo.frameSz.height - 1 - tgtPoint.y;
    
    Eigen::Vector4f pt3d;
    Plane::ground.getIntersection( pt3d, pt2d, scene.scnSfM.K, scene.scnSfM.vFrameData[guiParams.scnPos].M );
    
    //manipulate object
    if( guiParams.mMode==0 && !objects.empty())
    {
        ivAppObject *curObj = objects.getCurrentObject();
        curObj->objAnimation.objPose._trs(0) = pt3d(0);
        curObj->objAnimation.objPose._trs(1) = pt3d(1);
    }
    
    //manipulate light
    if( guiParams.mMode==1 )
    {
        scene.scnLight.pos(0) = pt3d(0);
        scene.scnLight.pos(1) = pt3d(1);
    }
    
    [self setRState_Manipulation];
}

//2 fingers scroll
- (void)scrollCallBack:(NSEvent *)theEvent
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppScene &scene = ivAppScene::instance();
    ivAppObjects &objects = ivAppObjects::instance();
    
    float angleRad = std::atan2([theEvent deltaY], [theEvent deltaX]);
    float angleDeg = angleRad * 180.0f / M_PI;
    
    const float angleThreshold = 30.0f;
    bool isHorizontal = std::abs(180.0f-angleDeg) < angleThreshold || std::abs(angleDeg) < angleThreshold;
    bool isVertical = std::abs(-90.0f-angleDeg) < angleThreshold || std::abs(90.0f-angleDeg) < angleThreshold;
    
    //manipulate object
    if( guiParams.mMode==0 && !objects.empty() )
    {
        ivAppObject *curObj = objects.getCurrentObject();
        if( isHorizontal )
        {
            float angleZ = M_PI * ( [theEvent deltaX] * 0.5f ) / 180.0f;
            Eigen::Quaternion<float> rotz;
            rotz = Eigen::Quaternion<float>( Eigen::AngleAxis<float>(  angleZ, Eigen::Vector3f::UnitZ() ) );
            curObj->objAnimation.objPose._rot = rotz * curObj->objAnimation.objPose._rot;
        }
        else if( isVertical )
        {
            float height = (float)[theEvent deltaY] / 300.0f;
            curObj->objAnimation.objPose._trs(2) -= height;
        }
        
        //constraint: on the ground
        if( curObj->objAnimation.objPose._trs(2)<0.0f )
            curObj->objAnimation.objPose._trs(2) = 0.0f;
    }
    
    //manipulate light
    if( guiParams.mMode==1 )
    {
        if( isVertical )
        {
            float height = (float)[theEvent deltaY] / 300.0f;
            scene.scnLight.pos(2) -= height;
        }
        if( scene.scnLight.pos(2)<0.0f )
            scene.scnLight.pos(2) = 0.0f;
    }
    
    //manipulate manipulation guidance
    //manipulate the unit sphere for manipulation guidance
    if ( guiParams.mMode==2  && !objects.empty() )
    {
        float delta = [theEvent deltaY];
        ivAppObject *curObj = objects.getCurrentObject();
        curObj->cameraDensity += delta;
        curObj->cameraDensity = std::max(  1.0f, curObj->cameraDensity);
        curObj->cameraDensity = std::min( 30.0f, curObj->cameraDensity);
    }
    
    [self setRState_Manipulation];
}

//scalling
- (void)zoomInOutCallBack:(NSEvent *)theEvent
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    ivAppScene &scene = ivAppScene::instance();
    ivAppObjects &objects = ivAppObjects::instance();
    
    float mag = [theEvent magnification];
    
    //manipulate object
    if( guiParams.mMode==0 && !objects.empty() )
    {
        ivAppObject *curObj = objects.getCurrentObject();
        if( mag+1.0f > 0.01f )
            curObj->objAnimation.objPose._scl *= 1.0f+mag;
    }
    
    //manipulate light
    if ( guiParams.mMode==1 )
    {
        const float tmp = 0.01;
        scene.scnLight.lightIntensity += mag>0.0f?tmp:-tmp;
        std::max( scene.scnLight.lightIntensity, 0.0f );
        std::min( scene.scnLight.lightIntensity, 1.0f );
    }
    
    //manipulate the unit sphere for manipulation guidance
    if ( guiParams.mMode==2  && !objects.empty() )
    {
        ivAppObject *curObj = objects.getCurrentObject();
        if( mag+1.0f > 0.01f )
            curObj->mGuidePose._scl *= 1.0f+mag;
    }
    
    [self setRState_Manipulation];
}

#pragma mark - Keyboard Call Back

//keyboard call back function
- (void) keyDownCallBack:(NSEvent *)theEvent
{
    NSString *characters = [theEvent characters];
    if ([characters length])
    {
        char character = [characters characterAtIndex:0];
        ivAppGuiParams *guiParams = [ivAppGuiParams instance];

        if( character == 'l' )
            guiParams.mMode = 1;
        if( character == 'm' )
            guiParams.mMode = 2;
    }
}
- (void) keyUpCallBack:(NSEvent *)theEvent
{
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams.mMode = 0;
    
    ivAppObjects &objects = ivAppObjects::instance();
    if( objects.empty() )
        return;
    ivAppObject *curObj = objects.getCurrentObject();
    
    NSString *characters = [theEvent characters];
    if ([characters length])
    {
        char character = [characters characterAtIndex:0];
        
        //push the object pose
        if( character == 'a' )
            curObj->objAnimation.push();
        
        //pop the object pose
        if( character == 'q' )
            curObj->objAnimation.pop();
        
        //clear the object pose
        if( character == 'c' )
            curObj->objAnimation.reset();
        
        //setup the source object motion
        if( character == 'o' )
            curObj->objAnimation.setObjMotion( curObj->objSfM );
        
        //estimate the bezier path
        if( character == 'a' || character == 'q' || character == 'c' || character == 'r' )
        {
            float BZR_scale = curObj->objParams.BZR_scale;
            int BZR_interval = curObj->objParams.BZR_interval;
            curObj->objAnimation.setAniParams(BZR_interval, BZR_scale);
            curObj->objAnimation.estimateObjAnimation();
            
            [self setupAnimation];
        }
        
        [self setRState_Manipulation];
    }
}

@end
