//
//  ivAppParameters.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivAppParameters.h"

@implementation ivAppParameters

#pragma mark - variables conversion

//display object params
-(NSMutableDictionary *) getObjParams
{
    ivAppObjects &objects = ivAppObjects::instance();

    NSMutableDictionary* parameters;
    parameters =[NSMutableDictionary dictionaryWithObjectsAndKeys:nil];
    
    if( !objects.empty() )
    {
    ivAppObject *obj = objects.getCurrentObject();
    ivAppObjParams &params = obj->objParams;
    parameters =[NSMutableDictionary dictionaryWithObjectsAndKeys:
                 [NSNumber numberWithFloat: params.OSG_oriDiff    ], @"OSG_oriDiff",
                 [NSNumber numberWithInt:   params.OSG_frmT       ], @"OSG_frmT",
                 [NSNumber numberWithInt:   params.NRM_nnum       ], @"NRM_nnum",
                 [NSNumber numberWithBool:  params.NRM_label      ], @"NRM_label",
                 [NSNumber numberWithBool:  params.CLSTR_KM_MS    ], @"CLSTR_KM_MS",
                 [NSNumber numberWithInt:   params.CLSTR_cnum     ], @"CLSTR_cnum",
                 [NSNumber numberWithFloat: params.CLSTR_nweight  ], @"CLSTR_nweight",
                 [NSNumber numberWithFloat: params.CLSTR_width    ], @"CLSTR_width",
                 [NSNumber numberWithFloat: params.CLSTR_sharedW  ], @"CLSTR_sharedW",
                 [NSNumber numberWithFloat: params.CLSTR_edgeDist ], @"CLSTR_edgeDist",
                 [NSNumber numberWithFloat: params.CLSTR_edgeOrnt ], @"CLSTR_edgeOrnt",
                 [NSNumber numberWithFloat: params.FS_simW        ], @"FS_simW",
                 [NSNumber numberWithInt:   params.WRP_gridSize   ], @"WRP_gridSize",
                 [NSNumber numberWithBool:  params.WRP_rgrid      ], @"WRP_rgrid",
                 [NSNumber numberWithFloat: params.WRP_ftrW       ], @"WRP_ftrW",
                 [NSNumber numberWithFloat: params.WRP_simW       ], @"WRP_simW",
                 [NSNumber numberWithFloat: params.WRP_edgeW      ], @"WRP_edgeW",
                 [NSNumber numberWithFloat: params.WRP_coW        ], @"WRP_coW",
                 [NSNumber numberWithInt:   params.WRP_eintrvl    ], @"WRP_eintrvl",
                 [NSNumber numberWithFloat: params.WRP_btmT       ], @"WRP_btmT",
                 [NSNumber numberWithFloat: params.WRP_btmW       ], @"WRP_btmW",
                 [NSNumber numberWithFloat: params.CLG_alpha      ], @"CLG_alpha",
                 [NSNumber numberWithFloat: params.CLG_feature    ], @"CLG_feature",
                 [NSNumber numberWithFloat: params.CLG_dstrn      ], @"CLG_dstrn",
                 [NSNumber numberWithFloat: params.CLG_sm_edge    ], @"CLG_sm_edge",
                 [NSNumber numberWithFloat: params.CLG_sm_clr     ], @"CLG_sm_clr",
                 [NSNumber numberWithFloat: params.CLG_sm_grad    ], @"CLG_sm_grad",
                 [NSNumber numberWithInt:   params.CLG_tm_fnum    ], @"CLG_tm_fnum",
                 [NSNumber numberWithFloat: params.CLG_tm_alpha   ], @"CLG_tm_alpha",
                 [NSNumber numberWithFloat: params.CLG_tm_w       ], @"CLG_tm_w",
                 [NSNumber numberWithInt:   params.CLG_edgeRange  ], @"CLG_edgeRange",
                 [NSNumber numberWithBool:  params.CLG_usePoisson ], @"CLG_usePoisson",
                 [NSNumber numberWithInt:   params.S2D_gridSz     ], @"S2D_gridSz",
                 [NSNumber numberWithFloat: params.S2D_dataW      ], @"S2D_dataW",
                 [NSNumber numberWithFloat: params.S2D_smthW      ], @"S2D_smthW",
                 [NSNumber numberWithFloat: params.SHDW_gain      ], @"SHDW_gain",
                 [NSNumber numberWithFloat: params.SHDW_alpha     ], @"SHDW_alpha",
                 [NSNumber numberWithInt:   params.SHDW_bKSz      ], @"SHDW_bKSz",
                 [NSNumber numberWithFloat: params.SHDW_bsigma    ], @"SHDW_bsigma",
                 [NSNumber numberWithFloat: params.BZR_scale      ], @"BZR_scale",
                 [NSNumber numberWithInt:   params.BZR_interval   ], @"BZR_interval",
                 [NSNumber numberWithFloat: params.LT_kd          ], @"LT_kd",
                 nil];
    }

    return parameters;
}

//save obj params
-(void) setObjParams:(NSMutableDictionary *)parameters
{
    ivAppObjects &objects = ivAppObjects::instance();
    
    if( !objects.empty() )
    {
        ivAppObject *obj = objects.getCurrentObject();
        ivAppObjParams &params = obj->objParams;
        
        params.OSG_oriDiff    = [[parameters objectForKey:@"OSG_oriDiff"    ] floatValue];
        params.OSG_frmT       = [[parameters objectForKey:@"OSG_frmT"       ]   intValue];
        params.NRM_nnum       = [[parameters objectForKey:@"NRM_nnum"       ]   intValue];
        params.NRM_label      = [[parameters objectForKey:@"NRM_label"      ]  boolValue];
        params.CLSTR_KM_MS    = [[parameters objectForKey:@"CLSTR_KM_MS"    ]  boolValue];
        params.CLSTR_cnum     = [[parameters objectForKey:@"CLSTR_cnum"     ]   intValue];
        params.CLSTR_nweight  = [[parameters objectForKey:@"CLSTR_nweight"  ] floatValue];
        params.CLSTR_width    = [[parameters objectForKey:@"CLSTR_width"    ] floatValue];
        params.CLSTR_sharedW  = [[parameters objectForKey:@"CLSTR_sharedW"  ] floatValue];
        params.CLSTR_edgeDist = [[parameters objectForKey:@"CLSTR_edgeDist" ] floatValue];
        params.CLSTR_edgeOrnt = [[parameters objectForKey:@"CLSTR_edgeOrnt" ] floatValue];
        params.FS_simW        = [[parameters objectForKey:@"FS_simW"        ] floatValue];
        params.WRP_gridSize   = [[parameters objectForKey:@"WRP_gridSize"   ]   intValue];
        params.WRP_rgrid      = [[parameters objectForKey:@"WRP_rgrid"      ]  boolValue];
        params.WRP_ftrW       = [[parameters objectForKey:@"WRP_ftrW"       ] floatValue];
        params.WRP_simW       = [[parameters objectForKey:@"WRP_simW"       ] floatValue];
        params.WRP_edgeW      = [[parameters objectForKey:@"WRP_edgeW"      ] floatValue];
        params.WRP_coW        = [[parameters objectForKey:@"WRP_coW"        ] floatValue];
        params.WRP_eintrvl    = [[parameters objectForKey:@"WRP_eintrvl"    ]   intValue];
        params.WRP_btmT       = [[parameters objectForKey:@"WRP_btmT"       ] floatValue];
        params.WRP_btmW       = [[parameters objectForKey:@"WRP_btmW"       ] floatValue];
        params.CLG_alpha      = [[parameters objectForKey:@"CLG_alpha"      ] floatValue];
        params.CLG_feature    = [[parameters objectForKey:@"CLG_feature"    ] floatValue];
        params.CLG_dstrn      = [[parameters objectForKey:@"CLG_dstrn"      ] floatValue];
        params.CLG_sm_edge    = [[parameters objectForKey:@"CLG_sm_edge"    ] floatValue];
        params.CLG_sm_clr     = [[parameters objectForKey:@"CLG_sm_clr"     ] floatValue];
        params.CLG_sm_grad    = [[parameters objectForKey:@"CLG_sm_grad"    ] floatValue];
        params.CLG_tm_fnum    = [[parameters objectForKey:@"CLG_tm_fnum"    ]   intValue];
        params.CLG_tm_alpha   = [[parameters objectForKey:@"CLG_tm_alpha"   ] floatValue];
        params.CLG_tm_w       = [[parameters objectForKey:@"CLG_tm_w"       ] floatValue];
        params.CLG_edgeRange  = [[parameters objectForKey:@"CLG_edgeRange"  ]   intValue];
        params.CLG_usePoisson = [[parameters objectForKey:@"CLG_usePoisson" ]  boolValue];
        params.S2D_gridSz     = [[parameters objectForKey:@"S2D_gridSz"     ]   intValue];
        params.S2D_dataW      = [[parameters objectForKey:@"S2D_dataW"      ] floatValue];
        params.S2D_smthW      = [[parameters objectForKey:@"S2D_smthW"      ] floatValue];
        params.SHDW_gain      = [[parameters objectForKey:@"SHDW_gain"      ] floatValue];
        params.SHDW_alpha     = [[parameters objectForKey:@"SHDW_alpha"     ] floatValue];
        params.SHDW_bKSz      = [[parameters objectForKey:@"SHDW_bKSz"      ]   intValue];
        params.SHDW_bsigma    = [[parameters objectForKey:@"SHDW_bsigma"    ] floatValue];
        params.BZR_scale      = [[parameters objectForKey:@"BZR_scale"      ] floatValue];
        params.BZR_interval   = [[parameters objectForKey:@"BZR_interval"   ]   intValue];
        params.LT_kd          = [[parameters objectForKey:@"LT_kd"          ] floatValue];
        
        if( params.SHDW_bKSz%2==0 )
            params.SHDW_bKSz++;
        
        params.CLG_tm_fnum = std::max( params.CLG_tm_fnum, 0 );
    }
}

#pragma mark - tableview for parameters

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
    NSMutableDictionary *parameters = [self getObjParams];
    return [parameters count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    NSMutableDictionary *parameters = [self getObjParams];
    NSArray *keys = [[parameters allKeys] sortedArrayUsingSelector: @selector(compare:)];
    
    if ([[tableColumn identifier] isEqualTo:@"param"])
        return [keys objectAtIndex:row];
    else if ([[tableColumn identifier] isEqualTo:@"value"])
        return [parameters objectForKey:[keys objectAtIndex:row]];
    
    return 0;
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)value forTableColumn:(NSTableColumn *)column row:(NSInteger)row
{
    NSMutableDictionary *parameters = [self getObjParams];
    NSArray *keys = [[parameters allKeys] sortedArrayUsingSelector: @selector(compare:)];

    [parameters setObject:value forKey:[keys objectAtIndex:row]];
    [self setObjParams:parameters];
    [tableView reloadData];
}


@end
