//
//  ivOglObjSrcView.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglObjSrcView.h"

@implementation ivOglObjSrcView

- (BOOL) acceptsFirstResponder{return YES;}
- (BOOL) becomeFirstResponder{return YES;}
- (BOOL) resignFirstResponder{return YES;}

//initialization
-(void) awakeFromNib
{
    //Enable dropping files on the view
    [self registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
    
    //set the window as the first responder
    [[self window] setAcceptsMouseMovedEvents:YES];
    [[self window] makeFirstResponder:self];
    
    //keep drawing...
    NSTimer *updateTimer = [NSTimer timerWithTimeInterval:1.0f/30.0f target:self selector:@selector(idle:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:updateTimer forMode:NSDefaultRunLoopMode];
    
    [self reset];
}

//redraw function
- (void)idle:(NSTimer*)timer
{
    [self setNeedsDisplay:YES];
}

//reset parameters
-(void)reset
{
    
}

//openGL initialization
- (void) prepareOpenGL
{
    Utils_Ogl::InitGL();
}

- (void)drawRect:(NSRect)dirtyRect
{
    glViewport(0,0,OSZ_WIDTH/2,OSZ_HEIGHT/2);
    ivAppGodView &godView = ivAppGodView::instance();
    godView.drawGodView();
    glSwapAPPLE();
}

- (void)mouseUp:(NSEvent *)theEvent
{
    ivAppGodView &godView = ivAppGodView::instance();
    godView.motionAxis = !godView.motionAxis;
}

- (void)rightMouseUp:(NSEvent *)theEvent
{
    ivAppGodView &godView = ivAppGodView::instance();
    godView.srcAxis = !godView.srcAxis;
}

- (void)scrollWheel:(NSEvent *)theEvent
{
    const float speedControl = .5f;
    ivAppGodView &godView = ivAppGodView::instance();
    
    godView.alpha -= M_PI * ( [theEvent deltaX]/OSZ_WIDTH  ) * speedControl;
    if( godView.alpha >= 2*M_PI )
        godView.alpha -= 2*M_PI;
    if( godView.alpha < 0.0 )
        godView.alpha += 2*M_PI;

    float angleSafety = M_PI/36;
    godView.beta += M_PI * ( [theEvent deltaY]/OSZ_HEIGHT ) * speedControl;
    if( godView.beta >= M_PI/2-angleSafety )
        godView.beta = M_PI/2-angleSafety;
    if( godView.beta < angleSafety )
        godView.beta = angleSafety;
}

//scale
- (void)magnifyWithEvent:(NSEvent *)theEvent
{
    float mag = [theEvent magnification];
    ivAppGodView &godView = ivAppGodView::instance();
    
    godView.targetScale *= 1.0f + mag;
    if( godView.targetScale < 0.1f )
        godView.targetScale = 0.1f;
    if( godView.targetScale > 10.0f )
        godView.targetScale = 10.0f;
}

@end
