//
//  ivOglView+IBActions.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView+IBActions.h"

@implementation ivOglView (IBActions)

//redraw callback
-(IBAction)redraw:(id)sender
{
    [self setNeedsDisplay:YES];
    [[self objSrcView] setNeedsDisplay:YES];
}

#pragma mark - View Call Back

//setup animation type
-(IBAction)setupAnimation:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    
    ivAppGuiParams *guiParams = [ivAppGuiParams instance];
    guiParams.aniPos = 0;
    [driver setAniPose];
    [driver setupAnimation];
    [self redraw:sender];
}

//set scene/object/animation pose
-(IBAction)setScnPose:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setRState_Manipulation];
    [self redraw:sender];
}
-(IBAction)setObjPose:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setObjPose];
    [driver setRState_Manipulation];
    [self redraw:sender];
}
-(IBAction)setAniPose:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setAniPose];
    [driver setRState_Manipulation];
    [self redraw:sender];
}

//switch to the scene
-(IBAction)gotoScene:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver gotoScene];
    [self redraw:sender];
}

-(IBAction)addObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver addObject];
    [self redraw:sender];
}
-(IBAction)rmCurObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver rmCurObject];
    [self redraw:sender];
}
-(IBAction)duplicateObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver duplicateObject];
    [self redraw:sender];
}
-(IBAction)nextObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver nextObject];
    [self redraw:sender];
}
-(IBAction)prevObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver prevObject];
    [self redraw:sender];
}

//reset object
-(IBAction)resetObject:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver resetObject];
    [self redraw:sender];
}

//select frames
-(IBAction)selectFrames:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver selectFrames];
    [self redraw:sender];
}

//process
-(IBAction)process:(id)sender
{
    isProcessing = YES;
    
    ivAppDriver *driver = [ivAppDriver instance];
    [driver process];
    
    isProcessing = NO;
    [self redraw:sender];
}

#pragma mark - Menu Call Back Render

// Render : set rendering mode
-(IBAction)setRState_Manipulation:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setRState_Manipulation];
    [self redraw:sender];
}

-(IBAction)setRState_Structure:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setRState_Structure];
    [self redraw:sender];
}

-(IBAction)setRState_Results:(id)sender
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver setRState_Results];
    [self redraw:sender];
}

#pragma mark - Menu Call Back Exporting

// Export: exporting results
-(IBAction)captureImage:(id)sender
{
    if( isCapturing )
        return;
    
    //get out path
    NSSavePanel *savePanel = [NSSavePanel savePanel];
    savePanel.title = @"Capture Image..";
    NSString* imgExt = [NSString stringWithUTF8String:"png"];
    NSArray *allowedTypes = [NSArray arrayWithObjects:imgExt,nil];
    [savePanel setAllowedFileTypes:allowedTypes];
    NSInteger status = [savePanel runModal];
    if( status != NSModalResponseOK )
        return;
    outPath = [[[savePanel URL] path] UTF8String];
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"RGB"];
    [alert addButtonWithTitle:@"GRAYSCALE"];
    [alert addButtonWithTitle:@"RGBA"];
    [alert setMessageText:@"Channel Size"];
    [alert setInformativeText:@""];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    switch ( [alert runModal] )
    {
        case NSAlertFirstButtonReturn:
            cn=3;
            break;
        case NSAlertSecondButtonReturn:
            cn=1;
            break;
        default:
            cn=4;
            break;
    }
    
    isCapturing = YES;
    [self redraw:sender];
}

//GUI prompts
- (double)inputFPS: (NSString *)defaultValue
{
    NSTextField *input = [[NSTextField alloc] initWithFrame:NSMakeRect(0, 0, 200, 24)];
    [input setStringValue:defaultValue];

    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Please Enter FPS"];
    [alert setInformativeText:@""];
    [alert setAlertStyle:NSWarningAlertStyle];
    [alert setAccessoryView:input];
    
    if ( [alert runModal] != NSAlertFirstButtonReturn )
    {
        [input validateEditing];
        return [input doubleValue];
    }

    return 0.0;
}

-(IBAction)exportVideo:(id)sender
{
    if( isExportingVideo )
        return;
    
    NSSavePanel *savePanel = [NSSavePanel savePanel];
    savePanel.title = @"Export Video..";
    NSString *movExt = [NSString stringWithUTF8String:"mov"];
    NSArray *allowedTypes = [NSArray arrayWithObjects:movExt,nil];
    [savePanel setAllowedFileTypes:allowedTypes];
    NSInteger status = [savePanel runModal];
    if( status != NSModalResponseOK )
        return;
    outPath = [[[savePanel URL] path] UTF8String];
    
    fps = [self inputFPS:@"30"];
    if( !fps )
        return;

    isExportingVideo = YES;
    [self redraw:sender];

}

@end
