//
//  ivOglView.mm
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView.h"

@implementation ivOglView

- (BOOL) acceptsFirstResponder{return YES;}
- (BOOL) becomeFirstResponder{return YES;}
- (BOOL) resignFirstResponder{return YES;}

//initialization
-(void) awakeFromNib
{
    //Enable dropping files on the view
    [self registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
    
    //set the window as the first responder
    [[self window] setAcceptsMouseMovedEvents:YES];
    [[self window] makeFirstResponder:self];
    
    //keep drawing...
    NSTimer *updateTimer = [NSTimer timerWithTimeInterval:1.0f/30.0f target:self selector:@selector(idle:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:updateTimer forMode:NSDefaultRunLoopMode];
    
    isProcessing = NO;
    isCapturing = NO;
    isExportingVideo = NO;
}

//redraw function
- (void)idle:(NSTimer*)timer
{
    [self setNeedsDisplay:YES];
    [[self objSrcView] setNeedsDisplay:YES];
}

//openGL initialization
- (void) prepareOpenGL
{
    Utils_Ogl::InitGL();
}

//rendering
- (void)drawRect:(NSRect)dirtyRect
{
    if( isProcessing )
        return;
    
    glViewport(0,0,OSZ_WIDTH,OSZ_HEIGHT);
    ivAppDriver *driver = [ivAppDriver instance];
    ivAppRenderer &renderer = ivAppRenderer::instance();
    
    if( !isCapturing && !isExportingVideo )
        renderer.render();
    
    //capturing
    if( isCapturing )
    {
        [driver captureImage:outPath chnNum:cn];
        isCapturing = NO;
    }
    
    //exporting video
    if( isExportingVideo )
    {
        [driver exportVideo:outPath FPS:fps];
        isExportingVideo = NO;
    }
    
    glSwapAPPLE();
}

#pragma mark - Mouse Call Back

//right click
- (void)rightMouseUp:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver nextObject];
}

//drag the mouse
- (void)mouseDragged:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    [driver dragCallBack:curPoint];
}

//2 fingers scroll
- (void)scrollWheel:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver scrollCallBack:theEvent];
}

//scale
- (void)magnifyWithEvent:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver zoomInOutCallBack:theEvent];
}

#pragma mark - Keyboard Call Back

//key up events
- (void) keyDown:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver keyDownCallBack:theEvent];
}
- (void) keyUp:(NSEvent *)theEvent
{
    ivAppDriver *driver = [ivAppDriver instance];
    [driver keyUpCallBack:theEvent];
}

@end
