//
//  ivAppData.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivAppData.h"

@implementation ivAppData

@synthesize framePos;
@synthesize frameMaxPos;

@synthesize objectVideos;
@synthesize selectedObject;

@synthesize isEdgeSnapping;
@synthesize isBackFaceCull;

- (id) init
{
    self = [super init];
    if ( self )
    {
        //start initialization
        NSOpenPanel *openPanel = [NSOpenPanel openPanel];
        openPanel.title = @"Open The Working Folder.";
        [openPanel setCanChooseDirectories:YES];
        [openPanel setCanChooseFiles:NO];
        NSInteger status = [openPanel runModal];
        if( status != NSModalResponseOK )
            exit(EXIT_SUCCESS);
        
        std::string wPath = [[[[openPanel URL] path] stringByAppendingString:@"/"] UTF8String];
        chdir(wPath.c_str());
        
        
        //init the list of the input video
        objectVideos = [[NSMutableArray alloc] init];
        std::vector<std::string> dirs = Utils_DirList::parseDirList(ivDirParams::videoDir);
        std::vector<std::string>::iterator it = dirs.begin();
        for(;it!=dirs.end();it++)
            if( it->substr(0,3) != "SCN" )
                [objectVideos addObject:@(it->c_str())];
        
        selectedObject = @(dirs[0].c_str());
        
        rStates = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                   @"1", @"OBJ_VIDEO",  //object video
                   @"0", @"OBJ_EDGES",  //object video edges
                   @"1", @"OBJ_SfM",    //object sfm
                   @"1", @"USR_LBL2D",  //user labeling 2D
                   @"1", @"USR_EDGE",   //user labeling edgeMap
                   @"1", @"USR_LBL3D",  //user labeling 2D
                   nil];
        
        isReady = false;
        isEdgeSnapping = lblParams::isEdgeSnapping;
        isBackFaceCull = lblParams::isBackFaceCull;
    }
    return self;
}

//read object
-(void) readObject
{
    std::string tcName = [selectedObject UTF8String];
    
    //read input video and calc edges
    std::string videoPath = ivDirParams::getVideoPath(tcName);
    video.readVideo( videoPath, INT_MAX );
    canny.init(&video);
    
    //original structure from motion data
    std::string objSfMFileName = ivDirParams::getObjSfMPath(tcName);
    sfmData.readObjSfM( objSfMFileName );
    
    //user labeling correspondence
    usrLbl.init( &sfmData, &canny, tcName );
    
    isReady = true;
    self.framePos = 0;
    self.frameMaxPos = video.frameNum-1;
}

//render results
-(void) render
{
    //render switches
    bool OBJ_VIDEO = [[rStates objectForKey:@"OBJ_VIDEO"] boolValue];
    bool OBJ_EDGES = [[rStates objectForKey:@"OBJ_EDGES"] boolValue];
    bool OBJ_SfM   = [[rStates objectForKey:@"OBJ_SfM"  ] boolValue];
    bool USR_EDGE  = [[rStates objectForKey:@"USR_EDGE"] boolValue];
    bool USR_LBL2D = [[rStates objectForKey:@"USR_LBL2D"] boolValue];
    bool USR_LBL3D = [[rStates objectForKey:@"USR_LBL3D"] boolValue];

    
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    if( !isReady )
        return;
    
    //=====================================================================
    // 2D Rendering
    //=====================================================================
    glDisable(GL_DEPTH_TEST);
    
    if( OBJ_VIDEO && video.isReady )
        Utils_Ogl::renderMat( video.frames[framePos] );
    
    if( OBJ_EDGES && canny.isReady )
    {
        cv::Mat alphaEdge = canny.getAlphaEdge(framePos);
        Utils_Ogl::renderMat( alphaEdge );
    }
    
    glEnable(GL_DEPTH_TEST);
    
    //=====================================================================
    // 3D Rendering
    //=====================================================================
    
    if( OBJ_SfM )
    {
        frameData &fd_Scn = sfmData.vFrameData[ framePos ];
        Utils_Ogl::setMatrix( sfmData.width, sfmData.height, sfmData.K, fd_Scn.M );
        sfmData.render(framePos);
        Utils_Ogl::resetMatrix();
    }
    
    //=====================================================================
    // 2D Rendering
    //=====================================================================
    glDisable(GL_DEPTH_TEST);
    
    if( USR_LBL2D )
    {
        cv::Mat lbl = usrLbl.render2D(framePos);
        Utils_Ogl::renderMat( lbl );
    }

    if( USR_EDGE )
    {
        cv::Mat edgeMap = usrLbl.render2DEdge(framePos, 3.0f, false);
        Utils_Ogl::renderMat( edgeMap );
    }
    
    if( USR_LBL3D )
    {
        cv::Mat edgeMap = usrLbl.render2DEdge(framePos, 1.0f, true);
        Utils_Ogl::renderMat( edgeMap );
    }
    
    glEnable(GL_DEPTH_TEST);
}

@end
