//
//  ivAppUsrLabel.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__ivAppUsrLabel__
#define __ivModeling__ivAppUsrLabel__


#include "ivAppVideoCanny.h"
#include "ivAppSfM.h"
#include "Plane.h"
#include "ivAppObjPose.h"

#include "ivDirParams.h"
#include "lblPolygon2D.h"
#include "ivAppTvModel.h"

class ivAppUsrLabel
{
private:
    //edges
    ivAppVideoCanny *canny;
    
    //sfm data
    ivAppSfM *sfmData;
    
    //polygon labeling object
    lblPolygon2D lblPoly;
    
    //time varying model
    ivAppTvModel tvm;
    
    //the id of picked point
    int pickedPointID;
    
public:
    //the model pose for rendering
    ivAppObjPose modelPose;
    
    //the 2D position of the cursor
    Eigen::Vector2f cursor;
    
    //initialization
    void init( ivAppSfM *sfmData, ivAppVideoCanny *canny, std::string tcName );
    
    //add a point
    void addPoint(int framePos);
    
    //remove 2D points from polygon
    void rmPolygon2DPoint();
    
    //pickup/put a point
    void pickPoint(int framePos);
    
    //adjust picked point
    void adjustPickedPoint(int framePos);
    
    //cancel adjusting
    void cancelAdjusting(int framePos);
    
    //remove picked one ring
    void removePickedOneRing(int framePos);
    
    //remove picked primitive
    void removePickedPrim(int framePos);
    
    //goto next selected prim
    void nextPrim();
    
    //set primitive critical
    void setCritical(int framePos);

    //clear user labeling
    void clearCurrentUserLabeling(int framePos);
    
    //rendering status
    cv::Mat render2D(int framePos);
    
    //rendering edge
    cv::Mat render2DEdge(int framePos, float edgeWidth, bool optimized);
    
    //optimize the model points
    void doSBA();
    
    //read the model
    void readModel(std::string objFileName);
    
    //export the model
    void exportModel(std::string objFileName);
};

#endif /* defined(__ivModeling__ivAppUsrLabel__) */
