//
//  ivOglView+KeyMouEvents.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView+KeyMouEvents.h"

@implementation ivOglView (KeyMouEvents)

static BOOL justDrag = NO;

- (BOOL) acceptsFirstResponder{return YES;}
- (BOOL) becomeFirstResponder{return YES;}
- (BOOL) resignFirstResponder{return YES;}

//=================================================
// Mouse Events
//=================================================
//left mouse up event
- (void)mouseUp:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;
    
    if( justDrag )
    {
        justDrag = NO;
        return;
    }
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    
    ivAppSfM &sfmData = [self ivad]->sfmData;
    NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    int y = sfmData.height - 1 - curPoint.y;
    usrLbl.cursor << curPoint.x, y;
    
    int framePos = [self ivad].framePos;
    usrLbl.pickPoint( framePos );
}

//left mouse motion event
- (void)mouseMoved:(NSEvent *)theEvent
{
    [[self window] makeFirstResponder: self];
    
    if( ![self ivad]->isReady )
        return;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    
    ivAppSfM &sfmData = [self ivad]->sfmData;
    int framePos = [self ivad].framePos;
    NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    int y = sfmData.height - 1 - curPoint.y;
    
    //boundary check
    int ViewPortParams[4];
    glGetIntegerv(GL_VIEWPORT, ViewPortParams);
    if( curPoint.x<0 || y<0 || curPoint.x>=ViewPortParams[2] || y >=ViewPortParams[3]  )
        usrLbl.clearCurrentUserLabeling(framePos);
    else
    {
        //adjust the picked point
        usrLbl.cursor << curPoint.x, y;
        usrLbl.adjustPickedPoint( framePos );
    }
}

//right mouse up event
- (void)rightMouseUp:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;

    ivAppSfM &sfmData = [self ivad]->sfmData;
    NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    int y = sfmData.height - 1 - curPoint.y;
    usrLbl.cursor << curPoint.x, y;
    
    int framePos = [self ivad].framePos;
    usrLbl.addPoint( framePos );
}

//drag the mouse
- (void)mouseDragged:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;
    
    justDrag = YES;

    NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    
    int framePos = [self ivad].framePos;
    ivAppSfM &sfmData = [self ivad]->sfmData;

    Eigen::Vector2f pt2d;
    pt2d << curPoint.x, sfmData.height - 1 - curPoint.y;
    
    std::vector<float> zaxis{0.0f,0.0f,1.0f,0.0f};
    Plane ground;
    ground.init(zaxis);
    
    Eigen::Vector4f pt3d;
    ground.getIntersection(pt3d, pt2d, sfmData.K, sfmData.vFrameData[framePos].M );
    
    ivAppObjPose &pose = [self ivad]->usrLbl.modelPose;
    pose._trs(0) = pt3d(0);
    pose._trs(1) = pt3d(1);
}

//2 fingers scroll
- (void)scrollWheel:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;
    
    float angleRad = std::atan2([theEvent deltaY], [theEvent deltaX]);
    float angleDeg = angleRad * 180.0f / M_PI;
    
    const float angleThreshold = 30.0f;
    bool isHorizontal = std::abs(180.0f-angleDeg) < angleThreshold || std::abs(angleDeg) < angleThreshold;
    
    ivAppObjPose &pose = [self ivad]->usrLbl.modelPose;

    if( isHorizontal )
    {
        float angleZ = M_PI * ( [theEvent deltaX] * 0.5f ) / 180.0f;
        Eigen::Quaternion<float> rotz;
        rotz = Eigen::Quaternion<float>( Eigen::AngleAxis<float>(  angleZ, Eigen::Vector3f::UnitZ() ) );
        pose._rot = rotz * pose._rot;
    }
}

//scale
- (void)magnifyWithEvent:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;
    
    float mag = [theEvent magnification];
    
    ivAppObjPose &pose = [self ivad]->usrLbl.modelPose;

    pose._scl *= 1.0f+mag;
    if( pose._scl(0) < 0.1f )
        pose._scl = Eigen::Vector3f::Identity() * 0.1f;
    if( pose._scl(0) > 5.0f )
        pose._scl = Eigen::Vector3f::Identity() * 5.0f;
    
    [self setFrameSize:self.frame.size];
}

@end
