//
//  ivOglView.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView.h"

@implementation ivOglView

//rendering
- (void)drawRect:(NSRect)dirtyRect
{
    glViewport(0,0,OSZ_WIDTH,OSZ_HEIGHT);
    
    // the rendering process
    [[self ivad] render];
    
    glSwapAPPLE();
}

//the ibaction for redrawing
-(IBAction)redraw:(id)sender
{
    [self setNeedsDisplay:YES];
}

//the ibaction for redrawing
-(IBAction)readObject:(id)sender
{
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Load the case?"];
    [alert setInformativeText:@"It may take a long time."];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    if ( [alert runModal] == NSAlertFirstButtonReturn )
        return;

    [[self ivad] readObject];
    [self redraw:sender];
}

//the ibaction for reset object pose
-(IBAction)resetModelPose:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.modelPose.reset();
}

//read tv model
-(IBAction)readModel:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Read the model?"];
    [alert setInformativeText:@"NOTE: It will clear current labels."];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    if ( [alert runModal] == NSAlertFirstButtonReturn )
        return;
    
    std::string tcName = [[self ivad].selectedObject UTF8String];
    std::string tvmPath = ivDirParams::getTvMPath(tcName);
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.readModel( tvmPath );
}


//export labeled tv model
-(IBAction)exportModel:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Write the model?"];
    [alert setInformativeText:@"NOTE: It will overwrite current model file."];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    if ( [alert runModal] == NSAlertFirstButtonReturn )
        return;
    
    std::string tcName = [[self ivad].selectedObject UTF8String];
    std::string tvmPath = ivDirParams::getTvMPath(tcName);
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.exportModel( tvmPath );
}

//optimize the label points
-(IBAction)doSBA:(id)sender
{
    if( ![self ivad]->isReady )
        return;

    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.doSBA();
}

-(std::string) getDateString
{
    NSDate *currDate = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc]init];
    [dateFormatter setDateFormat:@"dd_MM_YY_HH_mm_ss"];
    NSString *dateString = [dateFormatter stringFromDate:currDate];
    return [dateString UTF8String];
}

//remove vertex one ring
-(IBAction)rmPrim:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    int framePos = [self ivad].framePos;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.rmPolygon2DPoint();
    usrLbl.removePickedPrim(framePos);
}

//remove one ring prims
-(IBAction)rmOneRingPrim:(id)sender
{
    int framePos = [self ivad].framePos;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.rmPolygon2DPoint();
    usrLbl.removePickedOneRing( framePos );
}

//switch edge snapping
-(IBAction)switchEdgeSnapping:(id)sender
{
    lblParams::isEdgeSnapping = !lblParams::isEdgeSnapping;
    [self ivad].isEdgeSnapping = lblParams::isEdgeSnapping;
    
    if( ![self ivad]->isReady )
        return;
    
    int framePos = [self ivad].framePos;
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.adjustPickedPoint( framePos );
}

//next neighbor prim
-(IBAction)nextNeighborPrim:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.nextPrim();
}

//cancel the adjusting
-(IBAction)cancelAdjusting:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    int framePos = [self ivad].framePos;
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.cancelAdjusting(framePos);
}

//set Critical
-(IBAction)setCritical:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    int framePos = [self ivad].framePos;
    ivAppUsrLabel &usrLbl = [self ivad]->usrLbl;
    usrLbl.setCritical(framePos);
}

//backface switch
-(IBAction)switchBackFace:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    lblParams::isBackFaceCull = !lblParams::isBackFaceCull;
    [self ivad].isBackFaceCull = lblParams::isBackFaceCull;
}

@end


