//
//  tvmRasterizer.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by sulaispig on 1/5/15.
//  Copyright (c) 2015 sulaispig. All rights reserved.
//

#ifndef __ivModeling__tvmRasterizer__
#define __ivModeling__tvmRasterizer__

#include "ivAppSfM.h"
#include "ivAppObjPose.h"
#include "ivAppTvModel.h"

//data for a pixel
class VER
{
public:
    //projected pt3d
    Eigen::Vector3f pt;

    //set data by interpolation
    void interpolate(const VER &V1, const VER &V2, int minV, int maxV, int v);
};

class tvmRasterizer
{
private:
    cv::Size imgSz;
    ivAppSfM *sfmData;
    cv::Mat colorBuffer;
    cv::Mat depthBuffer;
    
    bool isOutOfBoundary(int x, int y);
    void renderPixel( int x, int y, const VER &cur, bool renderColor );
    void renderHorizontalTrapezoid( const VER &ul, const VER &ur, const VER &bl, const VER &br );
    void renderTriangleFILL( VER v1, VER v2, VER v3 );
    void renderEdge( VER v1, VER v2 );
    
public:
    //init
    void init( ivAppSfM *sfmData );
    
    //raster the tv model at framePos
    cv::Mat raster(ivAppTvModel &tvm, ivAppObjPose &pose, int framePos);
};


#endif /* defined(__ivModeling__tvmRasterizer__) */
