//
//  ivOglView+KeyMouEvents.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView+KeyMouEvents.h"

@implementation ivOglView (KeyMouEvents)

- (BOOL) acceptsFirstResponder{return YES;}
- (BOOL) becomeFirstResponder{return YES;}
- (BOOL) resignFirstResponder{return YES;}

//=================================================
// Mouse Events
//=================================================
//left mouse up event
- (void)mouseUp:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;

    if( [[[self ivad]->rStates objectForKey:STROKE_KEY] boolValue] )
    {
        int framePos = [self ivad].framePos;
        ivAppStroke &stroke = [self ivad]->stroke;
        cv::Mat binStroke = stroke.getBinaryStroke();
        stroke.clear();
        
        [self ivad]->sfm_scn.collectGroundPoints(framePos, binStroke, (bool)[self ivad].isStroke );
    }
}

//left mouse motion event
- (void)mouseMoved:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;

    if( [[[self ivad]->rStates objectForKey:STROKE_KEY] boolValue] )
    {
        ivAppStroke &stroke = [self ivad]->stroke;
        NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
        stroke.curPoint.x = curPoint.x;
        stroke.curPoint.y = curPoint.y;
    }
}

//drag the mouse
- (void)mouseDragged:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;

    if( [[[self ivad]->rStates objectForKey:STROKE_KEY] boolValue] )
    {
        ivAppStroke &stroke = [self ivad]->stroke;
        NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
        stroke.curPoint.x = curPoint.x;
        stroke.curPoint.y = curPoint.y;
        stroke.draw([self ivad].framePos);
    }
    else
    {
        NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
        
        int framePos = [self ivad].framePos;
        
        Eigen::Vector2f pt2d;
        pt2d << curPoint.x, [self ivad]->video.frameSz.height - 1 - curPoint.y;
        
        std::vector<float> zaxis{0.0f,0.0f,1.0f,0.0f};
        Plane ground;
        ground.init(zaxis);
        
        Eigen::Vector4f pt3d;
        ground.getIntersection(pt3d, pt2d, [self ivad]->sfm_obj.K, [self ivad]->sfm_obj.vFrameData[framePos].M );
        
        [self ivad]->objPose._trs(0) = pt3d(0);
        [self ivad]->objPose._trs(1) = pt3d(1);
    }
}

//2 fingers scroll
- (void)scrollWheel:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;

    float angleRad = std::atan2([theEvent deltaY], [theEvent deltaX]);
    float angleDeg = angleRad * 180.0f / M_PI;
    
    const float angleThreshold = 30.0f;
    bool isHorizontal = std::abs(180.0f-angleDeg) < angleThreshold || std::abs(angleDeg) < angleThreshold;
    
    if( isHorizontal )
    {
        float angleZ = M_PI * ( [theEvent deltaX] * 0.5f ) / 180.0f;
        Eigen::Quaternion<float> rotz;
        rotz = Eigen::Quaternion<float>( Eigen::AngleAxis<float>(  angleZ, Eigen::Vector3f::UnitZ() ) );
        [self ivad]->objPose._rot = rotz * [self ivad]->objPose._rot;
    }
}

//scale
- (void)magnifyWithEvent:(NSEvent *)theEvent
{
    if( ![self ivad]->isReady )
        return;

    float mag = [theEvent magnification];
    [self ivad]->objPose._scl *= 1.0f + mag;
    if( [self ivad]->objPose._scl(0) < 0.001f )
        [self ivad]->objPose._scl.setConstant(0.001f);
    
    [self setFrameSize:self.frame.size];
}

@end
