//
//  ivOglView.m
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#import "ivOglView.h"

@implementation ivOglView

//rendering function
- (void)drawRect:(NSRect)dirtyRect
{
    glViewport(0,0,OSZ_WIDTH,OSZ_HEIGHT);
    
    [[self ivad] render];
    
    glSwapAPPLE();
}

//the ibaction for intput popup button
-(IBAction)reset:(id)sender
{
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Load the case?"];
    [alert setInformativeText:@"It may take a long time."];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    if ( [alert runModal] == NSAlertFirstButtonReturn )
        return;

    [[self ivad] reset];
    [self setNeedsDisplay:YES];
    [tableview reloadData];
}

//the ibaction for redrawing
-(IBAction)redraw:(id)sender
{
    [self setNeedsDisplay:YES];
}

//adjust stroke size
-(IBAction)adjustStrokeSize:(id)sender
{
    if( ![self ivad]->isReady )
        return;

    ivAppStroke &stroke = [self ivad]->stroke;
    stroke.strkRadius = [self ivad].strokeSize;
}

//normalize the object
-(IBAction)normalize:(id)sender
{
    if( ![self ivad]->isReady )
        return;
    
    ivAppAlgoObjNrmlztn nrmlz;
    nrmlz.execute(_ivad);
    
    ivAppAlgoSetState ss;
    ss.execute(_ivad);
    
    [self setNeedsDisplay:YES];
    [tableview reloadData];
}

//update normalization
-(IBAction)update:(id)sender
{
    if( ![self ivad]->isReady )
        return;

    ivAppAlgoObjNrmlztnUpdate update;
    update.execute(_ivad);
}

//output objsfm file
-(IBAction)outputObjSfM:(id)sender
{
    if( ![self ivad]->isReady )
        return;

    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Cancel"];
    [alert addButtonWithTitle:@"OK"];
    [alert setMessageText:@"Save the objsfm?"];
    [alert setInformativeText:@"It will overwrite original objsfm file."];
    [alert setAlertStyle:NSWarningAlertStyle];
    
    if ( [alert runModal] == NSAlertFirstButtonReturn )
        return;
    
    ivAppAlgoOutputObjSfM oosfm;
    oosfm.execute(_ivad);
}

-(std::string) getDateString
{
    NSDate *currDate = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc]init];
    [dateFormatter setDateFormat:@"dd_MM_YY_HH_mm_ss"];
    NSString *dateString = [dateFormatter stringFromDate:currDate];
    return [dateString UTF8String];
}

@end
