//
//  patchCollager.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__patchCollager__
#define __ivManipulation__patchCollager__

#include "Utils_Ocv.h"
#include "ivDirParams.h"
#include "GCoptimization.h"
#include "PoissonBlending.hpp"

class imgSet : public std::vector<cv::Mat>
{
public:
    cv::Mat imgCollage;
};

//collageData
class collageData
{
public:
    //the id & color for each cluster
    std::vector<int> patchIDs;
    std::vector<Eigen::Vector3f> colorIDs;

    imgSet patchSet;           //patches
    imgSet dataMapSet;         //data maps
    imgSet edgeMapSet;         //edge maps
    imgSet edgeCornerMapSet;   //edge corner maps
    
    //push the data
    void push_back(int patchID, const Eigen::Vector3f &colorID,
                   cv::Mat &patch, cv::Mat &dataMap,
                   cv::Mat &edgeMap, cv::Mat &edgeCornerMap);
    
    //clear
    void clear();
};

class patchCollager
{
protected:
    //image parameters
    int width, height;
    int num_pixels;
    
    //collage data
    collageData clgData;
    
    //output: labeling result of graphcut
    std::vector<int> labels;
    std::vector<cv::Mat> srcIDWeightMaps;
    
    //combine the imgSet
    cv::Mat collageImgSet(const imgSet &iset, float alpha=0.0f);

    //combine all the sets
    void collageAllSets();
    
    //initialization
    void init(int patchID, const Eigen::Vector3f &colorID,
              cv::Mat &patch0, cv::Mat &dataMap0,
              cv::Mat &edgeMap0, cv::Mat &edgeCornerMap0);

public:
    //if the collage is ready
    bool isReady();
    
    //clear all the data
    void clear();

    //initialize the collage size
    void initSize( int width, int height );
    
    //add other patches
    void add(int patchID, const Eigen::Vector3f &colorID,
             cv::Mat &patch, cv::Mat &weightMap, cv::Mat &distortionMap,
             cv::Mat &edgeMap, cv::Mat &edgeCornerMap, cv::Mat &temporalWeightMap,
             float featureWeight, float distortionWeight,
             float cDiffWeight, float gDiffWeight, float edgeWeight, float edgeAttenuR,
             float tmWeight );
    
    //calculate source ID weight map
    void calcSrcIDWeightMaps(float CLG_tm_alpha);
    
    //get ith src map
    void getSrcIDWeightMap(cv::Mat &srcidmap, int srcID);
    
    //blend the collaged images
    cv::Mat getBlendedCollage();
    
    //get the warped image which is colored in visible region
    cv::Mat getColoredImg(int imgID, float alpha);
    
    //get current collage
    cv::Mat getImgCollage(float alpha=0.2)
    {
        return this->collageImgSet( clgData.patchSet, alpha );
    }
    cv::Mat getEdgeMapCollage()
    {
        return this->clgData.edgeMapSet.imgCollage.clone();
    }
    cv::Mat getEdgeMapCornerCollage()
    {
        return this->clgData.edgeCornerMapSet.imgCollage.clone();
    }

    friend class patchCollager;
};

#endif /* defined(__ivManipulation__easyPatchCollager__) */
