//
//  Plane.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__Plane__
#define __ivModeling__Plane__

class Plane
{
private:
    //ax+by+cz=d
    float a,b,c,d;
    
public:
    Plane();
    Plane(float a, float b, float c, float d);
    ~Plane();
    
    const static Plane ground;
    
    //initialize by a quad using SVD
    
    void init(const std::vector<float> &params);
    
    //initialize by points using SVD
    // 4xn matrix, n = # of pts
    //return answer
    void init(const Eigen::MatrixXf &pts);
    
    //calculate the error to the points
    void calcError(Eigen::VectorXf &error,
                   const Eigen::MatrixXf &pts)const;
    
    //invert the plane parameters
    void inverseNormal();
    
    //get the normal of the plane
    void getNormal(Eigen::Vector3f &n)const;
    
    //get the intersection (pt3d) of the ray and the plane
    void getIntersection(Eigen::Vector4f &pt3d,
                         const Eigen::Vector2f &pt2d,
                         const Eigen::Matrix4f &K,
                         const Eigen::Matrix4f &M      )const;
    void getIntersection(Eigen::Vector4f &pt3d,
                         const Eigen::Vector4f &p,
                         const Eigen::Vector4f &v      )const;
    

    //get the depth t
    // X          xt
    // Y = invP * yt
    // Z           t
    // invP.row(3) = [0 0 0 1]
    float getDepth(const Eigen::Vector2f &pt2d,
                   const Eigen::Matrix4f &invP )const;
    
    //get the nearest point on the plane
    void getNearestPoint(Eigen::Vector4f &pt3dn,
                         const Eigen::Vector4f &pt3d)const;
    
    //get the smallest distance to the plane
    float getShortestDist(const Eigen::Vector4f &pt3d)const;
    
    //pcl plane estimation
    bool estimatePlane(pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud);
    
    //get the depth map the plane
    cv::Mat getDepthMap(const Eigen::Matrix4f &prjM, const cv::Size frameSz)const;
};


#endif /* defined(__ivModeling__Plane__) */
