//
//  lsqSparseToDenseMap.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __test__lsqSparseToDenseMap__
#define __test__lsqSparseToDenseMap__

#include "Utils_Ocv.h"

class lsqSparseToDenseMap
{
private:
    int width, height;
    int gridSz;
    int gridNumX, gridNumY;
    Eigen::Matrix<Eigen::Vector2f, Eigen::Dynamic, Eigen::Dynamic> gridVs;
    Eigen::Matrix<float, Eigen::Dynamic, Eigen::Dynamic> gridValues;
    
    //Point correspondence constraints and its weight
    std::vector< cv::Point2f > vs;
    std::vector< float > values;

    //get bilinear coeffs
    void getBilinearCoeffs(int &gi, int &gj, float &alpha, float &beta,
                           const cv::Point2f &v);
    
    //check boundary
    bool boundaryCheck(const cv::Point2f &v);
    
    //2D to 1D: index mapping
    int V2DtoV1D(int row, int col);
    
    //set matrix for value alignment
    void setValueAlignment(std::vector< Eigen::Triplet<float> > &A_trips,
                           Eigen::VectorXf &b, int &curRow, float fweight );

    //set value similarity
    void setSmlrty(std::vector< Eigen::Triplet<float> > &A_trips,
                   Eigen::VectorXf &b, int &curRow, float simWeight );
    
    //solve sparse linear system
    float solve(std::vector< Eigen::Triplet<float> > &A_trips,
                Eigen::VectorXf &b, int nrows, int ncols );
    
public:
    lsqSparseToDenseMap(int width, int height, int gridSz);
    ~lsqSparseToDenseMap();
    
    //add data
    void addValue(const cv::Point2f &v, float value);
    
    //go least square optimization
    void goLSQ(float valueW, float simW);
    
    //get the dense data map
    cv::Mat getDenseMap();
};

#endif /* defined(__test__lsqSparseToDenseMap__) */
