//
//  Utils_DirList.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "Utils_DirList.h"

//get the list of the folder names in curDir
std::vector<std::string> Utils_DirList::parseDirList(std::string curDir){
    std::vector<std::string> dirList;
    
    DIR* dir_point = opendir( curDir.c_str() );
    dirent* entry = readdir(dir_point);
    
    while (entry)
    {
        if (entry->d_type == DT_DIR)
        {
            std::string dname = entry->d_name;
            if (dname != "." && dname != "..")
            dirList.push_back( dname );
        }
        entry = readdir(dir_point);
    }
    
    return dirList;
}

//search files whose extension is curExt in curDir, dirs[i]+files[i] is the searched file.
bool checkFileName(std::string fname, std::string ext, bool caseSensitive){
    size_t flen = fname.length();
    size_t extlen = ext.length();
    if( flen < extlen )return false;
    std::string compareTarget = fname.substr( flen-extlen, extlen );
    
    for(size_t i=0;i< extlen;i++){
        char a = compareTarget[i];
        char b = ext[i];
        if( tolower(a) != tolower(b) && !caseSensitive )
        return false;
        if( a!=b && caseSensitive )
        return false;
    }
    
    return true;
}
void Utils_DirList::searchExtRecursively
(std::vector<std::string> &dirs, std::vector<std::string> &files, std::string curDir, std::string curExt, bool CaseSensitive)
{
    DIR* dir_point = opendir( curDir.c_str() );
    if(!dir_point)return;
    
    dirent* entry = readdir(dir_point);
    while (entry){
        //if it is a directory
        if (entry->d_type == DT_DIR)
        {
            std::string dname = entry->d_name;
            if (dname != "." && dname != "..")
            searchExtRecursively( dirs, files, curDir+"/"+dname, curExt, CaseSensitive );
        }
        //if it is a file
        else if (entry->d_type == DT_REG){
            std::string fname = entry->d_name;
            if( checkFileName(fname, curExt, CaseSensitive) )
            {
                dirs.push_back(curDir+"/");
                files.push_back(fname);
            }
        }
        entry = readdir(dir_point);
    }
}
