//
//  Utils_Ocv.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__Utils_Ocv__
#define __ivManipulation__Utils_Ocv__

#include <opencv2/opencv.hpp>

class Utils_Ocv {
private:
    cv::Mat colormap;
    
private:
    Utils_Ocv();
    ~Utils_Ocv(){}
    Utils_Ocv(const Utils_Ocv &);
	Utils_Ocv& operator=(const Utils_Ocv&);
	static Utils_Ocv inst;

public:
    static Utils_Ocv& instance(){return inst;}
    
    //get color
    void getColorBGR( cv::Vec3b &color, int value, int maxValue, int minValue );
    void getColorBGR( Eigen::Vector3f &color, int value, int maxValue, int minValue );
    void getColorRGB( cv::Vec3b &color, int value, int maxValue, int minValue );
    void getColorRGB( Eigen::Vector3f &color, int value, int maxValue, int minValue );
    
    //get alpha masked gradient
    cv::Mat getGradient(const cv::Mat &img);
    cv::Mat getGradSaliency(const cv::Mat &img);
    
    //get sum of channels
    cv::Mat sumAbsDiff(const cv::Mat &img1, const cv::Mat &img2);
    
    //append alpha channel (set 0 if fg is invisible)
    cv::Mat appendAlpha(const cv::Mat &img, const cv::Mat &msk, bool toEraseBackGround=true);
    
    //get alpha mask
    cv::Mat getAlphaMask(const cv::Mat &img);
    
    //visualize float mat
    void visualizeEnergy(const cv::Mat &img, std::string winName);
    
    //convert grayscale image to R/G/B/W
    void gray8UC1ToColor8UC4(cv::Mat &colorImg, const cv::Mat &grayImg, const Eigen::Vector4f &bgra);

    //get contour map
    cv::Mat getContourMap(const cv::Mat &grayImg, float strkSz);
    
    //get bounding box of the fg mask
    bool getFGBB(cv::Rect &fgbb, const cv::Mat &mask);
    
    //draw arrow
    void drawArrow(cv::Mat& img, cv::Point pStart, cv::Point pEnd, int len, int alpha,
                   cv::Scalar& color, int thickness, int lineType);
    
    //weighted Blurring
    cv::Mat weightedGaussianBlur
    ( const cv::Mat &img, const cv::Mat &bSize, float sigma );

    //horizontally align
    cv::Mat hAlignImages(cv::Mat &img1, cv::Mat &img2);
    
    //blend color with the image
    void blendColor(cv::Mat &img, const cv::Vec3b &color, float alpha);

    //shed color on the image
    void shedColor(cv::Mat &img, const cv::Vec3b &color);

    //255-rgb
    void invertRGB(cv::Mat &img);
    
    //draw importance map
    void drawImpWeight(cv::Mat &importanceMap, const cv::Point2f &pt, int hksz);
    

    // Farneback dense optical flow calculate and show in Munsell system of colors
    // Author : Zouxy
    // Date   : 2013-3-15
    // HomePage : http://blog.csdn.net/zouxy09
    // Email  : zouxy09@qq.com
    
    // API calcOpticalFlowFarneback() comes from OpenCV, and this
    // 2D dense optical flow algorithm from the following paper:
    // Gunnar Farneback. "Two-Frame Motion Estimation Based on Polynomial Expansion".
    // And the OpenCV source code locate in ..\opencv2.4.3\modules\video\src\optflowgf.cpp

    //used to visualize motion field
    void motionToColor(cv::Mat flow, cv::Mat &color);
};

class Utils_Color
{
public:
    static cv::Scalar WHITE;
    static cv::Scalar BLACK;
    static cv::Scalar GRAY;
    static cv::Scalar RED;
    static cv::Scalar GREEN;
    static cv::Scalar BLUE;
    
    static cv::Scalar WHITE_A;
    static cv::Scalar BLACK_A;
    static cv::Scalar GRAY_A;
    static cv::Scalar RED_A;
    static cv::Scalar GREEN_A;
    static cv::Scalar BLUE_A;
    
};

class VideoOutput{
	double fps;
	cv::Size frameSz;
	cv::VideoWriter outputVideo;
    
public:
    //=================================================
	// constructor & destructor
	//=================================================
	VideoOutput( std::string filename, cv::Size frameSz, double fps=30);
	~VideoOutput();
	
	//=================================================
	// interface functions
	//=================================================
	void writeFrame(cv::Mat &current_frame);
	void finishWriting();
};

#endif /* defined(__ivManipulation__Utils_Ocv__) */
