//
//  Utils_Ogl.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__Utils_Ogl__
#define __ivModeling__Utils_Ogl__

class Utils_Ogl {
public:
    //convert Mat to openGL texture ID
    static GLuint matToGL(const cv::Mat &img);
    
    //init function
    static void InitGL();
    
    //render a Mat in openGL
    static void renderMat(const cv::Mat &img);
    
    //read color buffer(GL_BGRA only)
    static cv::Mat readColorBuffer(int ltx,int lty,int w,int h);
    static cv::Mat readWholeBuffer();
    static cv::Size getViewPortSize();
    
    //set/reset projection & modelview by K & M & W & H
    static void setMatrix(int W, int H,
                   const Eigen::Matrix4f &K,
                   const Eigen::Matrix4f &M );
    //set/reset projection & modelview to Identity
    static void setMatrixIdentity();
    static void resetMatrix();
    
    //draw x,y,z axis
    static void drawAxis();
    
    //draw standard cube
    static void drawCube();
    
    //draw pyramid
    static void drawPyramid();
    
    //draw standard sphere
    static void drawSphere(int slices, int stacks);
    
    //draw XY plane
    static void drawGridXY(int xw, int yw);
    static void blueGridXY(int xw, int yw, float alpha=1.0f);
    static void whiteGridXY(int xw, int yw, float alpha=1.0f);
    static void redGridXY(int xw, int yw, float alpha=1.0f);
    
    //draw viewport border
    static void drawViewportBorder(float width, const Eigen::Vector3f &color);
    
    //draw background color
    static void drawBackgroundColor(float r, float g, float b);
    
    //Frame Buffer Object
    class FBO
    {
    private:
        //frame buffer size
        int width, height;
        
        //buffer id
        GLuint frameBuffer;
        GLuint colorRenderBuffer;
        GLuint depthRenderBuffer;
        
        bool isBuffersReady;
        
    public:
        //if the machine support FBO
        static bool supportFBO();

        //constructor
        FBO();
        
        //start fbo rendering
        bool start(int width, int height);
        
        //dump frame buffer in fbo
        cv::Mat readFrameBuffer(int channelNum);
        
        //write the frame to an image
        void writeFrameBuffer(std::string imgPath, int channelNum);
        
        //release fbo
        void end();
    };
};

#endif /* defined(__ivModeling__Utils_Ogl__) */