//
//  CoGEPWarper.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__CoGEPWarper__
#define __ivManipulation__CoGEPWarper__

#include "GEPWarper.h"

//cross image correspondence
class CrossImgCrr
{
public:
    int i,j;
    cv::Point2f vi, vj;
    float pWeight;
};

//Collaborative Warper
class CoGEPWarper
{
protected:
    //input
    //=================================
    //cross image correspondence
    std::vector<CrossImgCrr> cics;
    
    //the warpers for images
    std::vector<GEPWarper> gepws;

    //the weight of each warp
    std::vector<float> gepwWeights;
    //=================================
    
public:
    
    //clear all data
    void clear();
    
    //add gep warps
    void addGepWarp( GEPWarper &gepw, float gepwWeight );

    //get the ref of certain warp and weight
    GEPWarper& at(int i){ return gepws[i]; }
    float& weightAt(int i){ return gepwWeights[i]; }
    
    //cross-image positional constraints
    void addCoPointConstraint(int i, const cv::Point2f &vi,
                              int j, const cv::Point2f &vj, float pWeight);
    
    //fix warpweights, solve for grid meshes
    // return error values of each warp
    Eigen::VectorXf optimizeForWarps
    (float ftrWeight, float simWeight, float edgeWeight,
     float coWeight, float btmWeight);
};

#endif /* defined(__ivManipulation__CoGEPWarper__) */
