//
//  GEPWarper.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__GEPWarper__
#define __ivManipulation__GEPWarper__

#include "Warper.h"
#include "Utils_Ocv.h"

class Line
{
public:
    std::vector< cv::Point2f > evs_from;
    
    //line parameters: sin(alpha)*x - cos(alpha)*y = b
    float sinAlpha, cosAlpha, b;
};

class GEPWarper : public Warper
{
protected:
    //  \0/
    // 3 X 1
    //  /2\
    //the weight for each triangle in grids
    Eigen::Matrix<Eigen::Vector4f, Eigen::Dynamic, Eigen::Dynamic> gridWeights;
    
    //Edge Points
    std::vector< Line > lines;
    
    //calc quad similarity weight using saliency
    void setSaliencyWeight(const cv::Mat &saliency);
    
    //set matrix for similarity ( sum of diff of triangles in quad  )
    void setSmlrty(std::vector< Eigen::Triplet<float> > &A_trips, Eigen::VectorXf &b,
                   int &curRow, float simWeight );
    
    //update edge points/parameters after warping
    void updateEdgeData();
    
    // set edge straightness
    void setStrghtnss(std::vector< Eigen::Triplet<float> > &A_trips, Eigen::VectorXf &b,
                      int &curRow, float edgeWeight, int sumEdgePtNum );
    
public:
    
    GEPWarper(const cv::Mat &img, int gridSz);
    
    //add edge constraint
    void addEdgeConstraint(const cv::Point2f &ev1, const cv::Point2f &ev2, int interval,
                           float initAlpha=0.0f, float initB=0.0f);
    
    //get warping results
    //=================================================================
    
    //get warped feature map (white point, black background)
    cv::Mat getPointMap( cv::Size osz )const;
    
    //get warped edge map (white edge, black background)
    cv::Mat getEdgeMap( cv::Size osz, float edgeRange )const;

    //get warped edge corner map (white edge, black background)
    cv::Mat getEdgeCornerMap( cv::Size osz )const;

    
    //get importance map 32FC1 Mat
    cv::Mat getImportanceMap( cv::Size osz )const;
    
    //get warp distortion map 32FC1 Mat
    cv::Mat getDistortionMap( cv::Size osz )const;
    
    //=================================================================
    
    friend class CoGEPWarper;
};


#endif /* defined(__ivManipulation__GEPWarper__) */
