//
//  Warper.h
//  ivWarping
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivWarping__Warper__
#define __ivWarping__Warper__

#include "Utils_Ocv.h"
#include "Utils_Ogl.h"

class Warper
{
protected:
    //input image
    cv::Mat img;
    int width, height;
    
    //warping mesh
    int gridSz;
    int gridNumX, gridNumY;
    Eigen::Matrix<Eigen::Vector2f, Eigen::Dynamic, Eigen::Dynamic> gridOriVs;
    Eigen::Matrix<Eigen::Vector3f, Eigen::Dynamic, Eigen::Dynamic> gridVs;
    
    //Point correspondence constraints and its weight
    std::vector< cv::Point2f > vFroms;
    std::vector< cv::Point2f > vTos;
    std::vector< float > weights;
    float weightSum;
    
    std::vector< cv::Point2f > vFromsHarder;
    std::vector< cv::Point2f > vTosHarder;
    std::vector< float > weightsHarder;
    float weightHarderSum;
    
    //get bilinear coefficient
    void getBilinearCoeffs(int &gi, int &gj, float &alpha, float &beta, const cv::Point2f &v)const ;
    
    //check boundary
    bool boundaryCheck(const cv::Point2f &v)const ;
    
    //2D to 1D: index mapping
    int V2DtoV1D(int row, int col)const ;
    
    //set matrix for point correspondence
    void setPtCrrspndnc( std::vector< Eigen::Triplet<float> > &A_trips, Eigen::VectorXf &b, int &curRow, float fweight );
    void setHarderPtCrrspndnc( std::vector< Eigen::Triplet<float> > &A_trips, Eigen::VectorXf &b, int &curRow, float fweight );
    
    //solve sparse linear system, set the answer
    //return the average sum of difference of points before and after warping
    float solveGrids( std::vector< Eigen::Triplet<float> > &A_trips, Eigen::VectorXf &b, int nrows, int ncols );
    
public:
    Warper(const cv::Mat &img, int gridSz);
    
    //add point correspondence constraint
    void addPointConstraint( const cv::Point2f &vFrom, const cv::Point2f &vTo, float weight );
    void addHarderPointConstraint( const cv::Point2f &vFrom, const cv::Point2f &vTo, float weight );
    
    //remap the point to the warped space
    void remapPoint( const cv::Point2f &vFrom, cv::Point2f &vTo )const;
    
    //get the deformed image
    cv::Mat getDeformedImg(cv::Size osz, bool rgrid)const ;
    cv::Mat getDeformedImg(const cv::Mat &img, cv::Size osz, bool rgrid);
};

#endif /* defined(__ivWarping__Warper__) */
