//
//  ivAppVideo.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppVideo.h"

//=================================================
// read the video name inputVideo
// (destory original data)
//=================================================
bool ivAppVideo::readVideo(std::string inputVideo, int bufSize)
{
    this->frames.clear();
    
    cv::VideoCapture capture;
    capture.open(inputVideo.c_str());
    
    this->isReady = capture.isOpened();

    if( this->isReady )
    {
        //the size of input video
        this->frameSz = cv::Size((int) capture.get(CV_CAP_PROP_FRAME_WIDTH),
                                 (int) capture.get(CV_CAP_PROP_FRAME_HEIGHT) );
        
        //buffer all the video frames, check the frame number
        this->frameNum = 0;
        while(true)
        {
            if( this->frameNum>=bufSize )
                break;
            
            cv::Mat frame;
            capture >> frame;
            if(frame.empty())break;
            
            if( this->frameNum == 0 )
            {
                this->frames.push_back(frame);
                this->frameNum++;
            }
            
            this->frames.push_back(frame);
            this->frameNum++;
        }
        
        capture.release();
    }
    
    this->frameNum--;
    
    return this->isReady;
}

void ivAppVideo::calcMeanLuminance(bool useBilateralFilter)
{
    this->meanLuminance.resize( this->frameNum );
    
    // 10% of image height / color value range
    const double spatialSigma = 5;
    const double colorSigma = 100;
    
    for(int i=0;i<this->frameNum;i++)
    {
        cv::Mat frameLab;
        cv::cvtColor( this->frames[i], frameLab, CV_BGR2Lab );
        
        std::vector<cv::Mat> labChannels;
        cv::split( frameLab, labChannels );
        
        cv::Mat L = labChannels[0].clone();
        
        if( useBilateralFilter )
            cv::bilateralFilter( labChannels[0], L, -1, colorSigma, spatialSigma );

        this->meanLuminance[i] = cv::mean( L );
    }
}


