//
//  ivAppVideoCanny.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppVideoCanny.h"

//calculate canny edges
bool ivAppVideoCanny::init(ivAppVideo *vi)
{
    const int lowThreshold = 20;
    const int ratio = 3;
    const int kernel_size = 3;
    
    this->frameSz = vi->frameSz;
    this->frameNum = vi->frameNum;
    
    this->edges.resize( this->frameNum );
    
    for(int i=0;i<frameNum;i++)
    {
        cv::Mat src_gray;
        
        /// Convert the image to grayscale
        cv::cvtColor( vi->frames[i], src_gray, CV_BGR2GRAY );
        
        /// Reduce noise with a kernel 3x3
        cv::blur( src_gray, edges[i], cv::Size(3,3) );
        
        /// Canny detector
        cv::Canny( edges[i], edges[i], lowThreshold, lowThreshold*ratio, kernel_size );
    }
    
    this->isReady = true;
    return true;
}


//rendering the edge
cv::Mat ivAppVideoCanny::getAlphaEdge(int framePos)
{
    std::vector<cv::Mat> nchannels;

    for(int i=0;i<4;i++)
        nchannels.push_back( this->edges[framePos] );
    
    cv::Mat alphaEdgeMap;
    cv::merge( nchannels, alphaEdgeMap);
    return alphaEdgeMap;
}
