//
//  ivAppVideoEdges.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by sulaispig on 12/6/14.
//  Copyright (c) 2014 sulaispig. All rights reserved.
//

#include "ivAppVideoEdges.h"

FrameEdges::FrameEdges(cv::Mat &img)
{
    this->edges = CmCurveEx::getEdges(img, true);
}

cv::Mat FrameEdges::drawInROI(cv::Mat &roiMsk,
                              cv::Rect roi,
                              cv::Point curVec,
                              int lineLen,
                              float angle,
                              float lenRatio)
{
    //the angle between shoul be less than 10
    const float cosCheck = std::cos(angle/180.0f*3.14159f);
    
    //the target edges to be colloected
    cv::Mat lineMskAll = cv::Mat::zeros( roi.height, roi.width, CV_8UC1 );
    
    //for each edges
    for(size_t i=0;i<edges.size();i++)
    {
        //the properties of a line
        cv::Mat lineMsk;
        cv::Point2f lineVec;
        int ptNum;
        edges[i].drawLineInRoi(roiMsk, roi, lineMsk, lineVec, ptNum );
        
        //check slope
        float mCheck = std::abs( curVec.dot( lineVec ) ) > cosCheck;
        //check length
        float nCheck = (float)ptNum / lineLen > lenRatio;
        
        //check whether the edge is good, and then collect it
        if( mCheck && nCheck )
            lineMskAll |= roiMsk & lineMsk;
    }
    
    return lineMskAll;
}

ivAppVideoEdges::~ivAppVideoEdges()
{
    this->clear();
}

void ivAppVideoEdges::init(ivAppVideo *vi)
{
    int frameNum = vi->frameNum;
    this->ves.resize( frameNum );
    
    for( int i=0; i<frameNum; i++ )
    {
        cv::Mat curFrame = vi->frames[i];
        this->ves[i] = new FrameEdges( curFrame );
    }
}

void ivAppVideoEdges::clear()
{
    std::vector< FrameEdges* >::iterator it = this->ves.begin();
    for( ; it!=ves.end(); it++ )
        delete *it;
}

//get the edges of the frame pos
FrameEdges* ivAppVideoEdges::getFrameEdges( int pos )
{
    return this->ves[pos];
}

void ivAppVideoEdges::renderEdges(cv::Mat &drawing, int pos)
{
    std::vector< CmCurveEx::CEdge > edges = this->ves[pos]->edges;
    
    for (size_t i = 0; i < edges.size(); i++)
    {
        cv::Vec3b color(rand()%128+128,rand()%128+128,rand()%128+128);
        
        const std::vector<cv::Point> &pnts = edges[i].pnts;
        for (size_t j = 0; j < pnts.size(); j++)
            drawing.at<cv::Vec3b>(pnts[j]) = color;
    }
}
