//
//  ivAppTvEdges.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppTvEdges.h"

//tv model input
void ivAppTvEdges::readTVModel(std::string fileName, ivAppSfM *sfmData)
{
    ivAppTvModelData::readTVModel(fileName, sfmData);

    const int fNum = (int)sfmData->vFrameData.size();
    
    std::vector< tvEdge > vtvEdgeTmp;
    
    //for each primitive
    int primNum = (int)this->vtvPrim.size();
    for(int i=0;i<primNum;i++)
    {
        tvTriPrim &curPrim = this->vtvPrim[i];
        
        //calc visibility
        std::vector<bool> primVisi(fNum);
        for(int j=0;j<fNum;j++)
            primVisi[j] = curPrim.isFront(j);
        
        //collect edges
        int ptNum = (int)curPrim.polygon.size();
        for(int j=0;j<ptNum;j++)
        {
            int k = (j+1)%ptNum;
            
            tvEdge tve;
            vtvEdgeTmp.push_back(tve);
            vtvEdgeTmp.back().vids << curPrim.polygon[j], curPrim.polygon[k];
            vtvEdgeTmp.back().isVisible = primVisi;
        }
    }
    
    //post-process the edges
    this->vtvEdge.clear();
    int edgeNum = (int)vtvEdgeTmp.size();
    for(int i=0;i<edgeNum;i++)
    {
        tvEdge &tmp = vtvEdgeTmp[i];
        
        //check if the edge is duplicated
        bool isDuplicated = false;
        std::vector< tvEdge >::iterator it = vtvEdge.begin();
        for(;it!=vtvEdge.end();it++)
        {
            bool edgeEq1 = it->vids(0)==tmp.vids(0) && it->vids(1)==tmp.vids(1);
            bool edgeEq2 = it->vids(0)==tmp.vids(1) && it->vids(1)==tmp.vids(0);
            
            if( edgeEq1 || edgeEq2 )
            {
                isDuplicated = true;
                break;
            }
        }
        
        if( !isDuplicated )
            this->vtvEdge.push_back(tmp);
        else //merge the duplicated edges
            for(int j=0;j<fNum;j++)
                it->isVisible[j] = it->isVisible[j] || tmp.isVisible[j];
    }
}
