//
//  ivAppTvModel.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__ivAppTvModel__
#define __ivModeling__ivAppTvModel__

#include "ivAppTvModelData.h"

//time varying model
class ivAppTvModel : public ivAppTvModelData
{
protected:
    
    //find the front primitive that contain the point
    //(primitives ids are in ascending order)
    std::vector<int> findNeighborPrims(int ptid);
    std::vector<int> findFrontNeighborPrims(int framePos, int ptid);

    //remove isolated & useless(not connected) points
    void rmIsolatedPoints(std::vector<bool> &isIsolated);

    //unify overlapping points in different primitive
    void unifyPoint(int framePos, int ptid);
    
    //get selected primID
    int getPrimID(int framePos, int selectedPt);

    //remove the primitive
    void rmPrimitive(int primID);

public:
    unsigned primPointer;
    
    //clear all data
    virtual void init();
    
    //select a point, if the point doesn't exist return -1
    //the returned point is not notthisone
    int selectPoint(int framePos, const Eigen::Vector2f &p, int nothisone=-1 );
    
    //push_back a 2d point candidate to be a 3d point
    int pushPointCandidate(int fNum, int framePos, const Eigen::Vector2f &p);
    
    //pop_back a 2d point candidate
    void popPointCandidate(int framePos, int ptid);
    
    //get the pointer to certain 2d point
    Eigen::Vector2f* getPoint2D(int framePos, int ptid);

    
    //add a primitive
    void addPrimitive(int framePos, ivAppSfM *sfmData, lblPolygon2D &lblPoly);

    //update picked point & neighboring polygon
    void adjustOneRing(int ptid, int framePos, const Eigen::Vector2f &p,
                         lblPolygon2D &lblPolyAdjust, ivAppSfM *sfmData=NULL);
    
    //cancel the adjusting
    void cancelAdjusting(int framePos, ivAppSfM *sfmData, int selectedPt);
    
    //remove neighboring primitives
    void rmOneRingPrims(int framePos, int selectedPt);
    
    //remove the primitive
    void rmPrimitive(int framePos, int selectedPt);
    
    //set the critical of the selected prim
    void setPrimCritical(int framePos, int fNum, int selectedPt);
    
    
    //render by OCV
    void renderOCV(cv::Mat &canvas, int framePos, int selectedPt);
};

#endif /* defined(__ivModeling__ivAppTvModel__) */
