//
//  ivAppTvModelData.cpp
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#include "ivAppTvModelData.h"

//clear all data
void ivAppTvModelData::init()
{
    this->vtvPoint.clear();
    this->vtvPrim.clear();
}

//render by OGL(cover all of the frame buffer)
void ivAppTvModelData::renderOGL(int framePos, Eigen::Matrix4f &prjM, bool inOriginalFrame, bool optimized)
{
    //for each primitive
    int primNum = (int)vtvPrim.size();
    for(int i=0;i<primNum;i++)
    {
        //check by normal
        bool frontalCheck1 = !inOriginalFrame && this->vtvPrim[i].isFront(vtvPoint, prjM, framePos);
        //check by user labeling
        bool frontalCheck2 =  inOriginalFrame && this->vtvPrim[i].isFront(framePos);
        
        if( frontalCheck1 || frontalCheck2 )
        {
            if( optimized )
                this->vtvPrim[i].renderOGL( this->vtvPoint );
            else
                this->vtvPrim[i].renderOGL( this->vtvPoint, framePos );
        }
    }
}

//get user labeled edge map
cv::Mat ivAppTvModelData::getEdgeMap
(ivAppSfM *scnSfM, int framePos, Eigen::Matrix4f &trnsfrm,
 int tvmPos,float edgeWidth, bool inOriginalFrame, bool optimized)
{
    cv::Mat edgeMap = cv::Mat::zeros( scnSfM->height, scnSfM->width, CV_8UC1 );
    
    Utils_Ogl::FBO fboObject;
    bool isFboReady = fboObject.start( scnSfM->width, scnSfM->height );
    if( isFboReady )
    {
        Utils_Ogl::setMatrix( scnSfM->width, scnSfM->height, scnSfM->K, scnSfM->vFrameData[framePos].M );
        glMultMatrixf(trnsfrm.data());
        
        glPushAttrib( GL_CURRENT_BIT | GL_LINE_BIT | GL_ENABLE_BIT | GL_POLYGON_BIT );
        {
            Utils_Ogl::InitGL();
            glViewport(0,0,scnSfM->width,scnSfM->height);
            glLineWidth(edgeWidth);
            glDisable(GL_BLEND);
            
            //clear frame buffer and rendering and readColorBuffer..
            glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
            
            //rendering
            Eigen::Matrix4f prjM = scnSfM->K * scnSfM->vFrameData[framePos].M * trnsfrm;
            this->renderOGL(tvmPos, prjM, inOriginalFrame, optimized);
        }
        glPopAttrib();
        
        //get rgba frame
        edgeMap = fboObject.readFrameBuffer(1);
        
        Utils_Ogl::resetMatrix();
        
        fboObject.end();
    }
    
    return edgeMap;
}

//bundle adjustment for the primitives
void ivAppTvModelData::doSBA(const ivAppSfM *sfmData)
{
    ivAppBA baObj;
    baObj.optimizeStructure( this->vtvPoint, this->vtvPrim, sfmData );
}

//model IO
void ivAppTvModelData::readTVModel(std::string fileName, ivAppSfM *sfmData)
{
    //clear data
    this->init();
    
    std::ifstream ifs(fileName);
    
    if( !ifs.good() )
        return;
    
    int fNum = (int)sfmData->vFrameData.size();
    
    //tvpoints
    int tvPtNum;
    ifs >> tvPtNum;
    this->vtvPoint.resize(tvPtNum);
    
    for(int i=0;i<tvPtNum;i++)
    {
        this->vtvPoint[i].init(fNum);
        ifs >> this->vtvPoint[i];
    }
    
    //primitive
    int primNum;
    ifs >> primNum;
    this->vtvPrim.resize(primNum);
    
    for(int i=0;i<primNum;i++)
    {
        ifs >> this->vtvPrim[i];
        this->vtvPrim[i].digest(this->vtvPoint, sfmData);
    }
    
    ifs.close();
}

void ivAppTvModelData::exportTVModel(std::string fileName, int fNum)
{
    std::ofstream ofs(fileName);
    
    if( !ofs.good() )
        return;
    
    //output time varying points
    int tvPtNum = (int)this->vtvPoint.size();
    ofs << tvPtNum << std::endl;
    for(int i=0;i<tvPtNum;i++)
        ofs << this->vtvPoint[i];
    
    //output primitive
    int primNum = (int)this->vtvPrim.size();
    ofs << primNum << std::endl;
    for(int i=0;i<primNum;i++)
        ofs << this->vtvPrim[i];
    
    ofs.close();
}


