//
//  ivAppTvModelData.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivManipulation__ivAppTvModelData__
#define __ivManipulation__ivAppTvModelData__

#include "lblParams.h"
#include "tvPoint3D.h"
#include "tvTriPrim.h"

#include "Utils_Ocv.h"
#include "Utils_Ogl.h"
#include "ivAppSfM.h"

#include "ivAppBA.h"

class ivAppTvModelData
{
public:
    //time-varying point set
    tvPoint3DSet vtvPoint;
    
    //primitive set
    tvTriPrimSet vtvPrim;
    
    //clear all data
    virtual void init();
    
    //render by OGL(cover all of the frame buffer)
    void renderOGL(int framePos, Eigen::Matrix4f &prjM,
                   bool inOriginalFrame, bool optimized);
    
    //get user labeled edge map
    cv::Mat getEdgeMap(ivAppSfM *scnSfM, int framePos, Eigen::Matrix4f &trnsfrm,
                       int tvmPos, float edgeWidth,
                       bool inOriginalFrame, bool optimized);
    
    //bundle adjustment for the primitives
    void doSBA(const ivAppSfM *sfmData);
    
    //model IO
    void readTVModel(std::string fileName, ivAppSfM *sfmData);
    
    void exportTVModel(std::string fileName, int fNum);
};

#endif /* defined(__ivManipulation__ivAppTvModelData__) */
