//
//  lblPolygon2D.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__lblPolygon2D__
#define __ivModeling__lblPolygon2D__

#include "lblParams.h"
#include "Utils_Ocv.h"
#include "tvPoint3D.h"
#include "ivAppVideoCanny.h"

//labeling a polgon in a frame
class lblPolygon2D
{
private:
    ivAppVideoCanny *canny;
    
    //check if the polygon is simple
    bool isSimplePolygon();
    
public:
    //the labeled frame id
    int curFrame;
    
    //the 2D polygon points
    std::vector< Eigen::Vector2f > plgn2d;
    
    //the indices of corresponding tvpoint3d
    std::vector<int> prim;
    
    //if the labeling finishes
    bool isReady;
    
    //constructors
    lblPolygon2D(){this->reset();}
    lblPolygon2D(ivAppVideoCanny *canny){this->init(canny);}
    
    //reset
    void reset();
    //init
    void init(ivAppVideoCanny *canny);
    
    //add a point to polygon point set
    bool addPoint2D(tvPoint3DSet &vtvPoint, int framePos,
                    Eigen::Vector2f lblp, int pid, bool toSnapEdge );
    
    //remove the last point
    void rmPoint2D();
    
    //rendering
    void render2D(cv::Mat &canvas);
    
    //rendering
    void render2D_prev(cv::Mat &canvas, tvPoint3DSet &vtvPoint,
                       int framePos, Eigen::Vector2f lblp);
    
    //get polygon region (white CV_8UC1)
    cv::Mat getRegion(int width, int height);
};

#endif /* defined(__ivModeling__lblPolygon2D__) */
