//
//  tvPoint3D.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__tvPoint3D__
#define __ivModeling__tvPoint3D__

#include "ivAppSfM.h"
#include "openMVG/multiview/triangulation_nview.hpp"
#include "Plane.h"

//time varying 3D points and projection
//=========================================================
class tvPoint3D
{
    //get the index of key frames
    void getKeyFrameIDs(std::vector<int> &key);
    
public:
    //optimized point
    Eigen::Vector4f v3D;
    
    //time-varying point and projection
    std::vector<Eigen::Vector4f> tvv3D;
    std::vector<Eigen::Vector2f> tvv2D;
    
    //if the vertex is keypoint
    std::vector<bool> isKey;
    
    //if the 3d vertex is ready
    std::vector<bool> isReady3D;

    //if the tv 3D point is reliably estimated in the frame
    std::vector<bool> isReliable3D;
    
    
    //initialize the point
    void init( int timeLen );
    
    //recover the 2d point by projection
    void recoverPoint2D( int framePos, const Eigen::Matrix4f &prjM );
    
    //propagate keyframe points to non-keyframe ones
    void propagateTVPT3D(const ivAppSfM *sfmData);
    
    //3d points triangulation for unreliable points
    void triangulateTVPT3D(const ivAppSfM *sfmData);
    
    //io
    friend std::istream& operator>>(std::istream& is, tvPoint3D& tvpt);
    friend std::ostream& operator<<(std::ostream& os, const tvPoint3D& tvpt);
};

//point set
typedef std::vector<tvPoint3D> tvPoint3DSet;

#endif /* defined(__ivModeling__tvPoint3D__) */
