//
//  tvTriPrim.h
//  Interactive Videos: Plausible Video Editing using Sparse Structure Points
//
//  Created by Chia-Sheng Chang on 03/01/16.
//  Copyright (c) 2016 Chia-Sheng Chang. All rights reserved.
//

#ifndef __ivModeling__tvTriPrim__
#define __ivModeling__tvTriPrim__

#include "lblParams.h"
#include "Utils_Ocv.h"
#include "tvPoint3D.h"
#include "lblPolygon2D.h"
#include "Poly2Tri.h"

//general triangulated polygon
class tvTriPrim
{
private:
    //if the primitive is visisble
    std::vector<bool> isVisible;
    
    //critical frames(visible/invisible)
    std::vector<int> criticalFrames;
    std::vector<bool> isCritical;
    
    //calc visibility of the primitive in framePos by
    //cross product
    void calcVisibilityByCross(tvPoint3DSet &vtvPoint, int fNum);
    //critical labeling
    void calcVisibilityByCritical(tvPoint3DSet &vtvPoint, int fNum);
    
    //calculate if the primitive is visible
    void calcVisibility(tvPoint3DSet &vtvPoint, int fNum);
    
    //is prim 3d ready
    bool isReady3D(tvPoint3DSet &vtvPoint, int fNum);

public:
    //the frame where the polygon is labeled
    int originFrame;
    
    //polygon vertex order
    std::vector<int> polygon;
    
    //the polygon triangulation
    std::vector<Eigen::Vector3i> triangles;
    
    //check if the point is in the primitive
    bool ifContainPt(int ptid);
    
    //mark points as true or false
    void markVertex(std::vector<bool> &vertexMarks, bool mark);
    
    //constrainted delaunay triangulation on the polygon
    void cdTriangulate(tvPoint3DSet &vtvPoint);
    
    //point index remapping
    void remapIndex(const std::vector<int> &newPtInd);
    
    //tvpoint propagation & primitive triangulation
    void digest(tvPoint3DSet &vtvPoint, ivAppSfM *sfmData);
    
    //if the primitive is front in the frame
    //(used when in original video)
    bool isFront(int framePos)const;
    //(used when in novel view)
    bool isFront(const tvPoint3DSet &vtvPoint, const Eigen::Matrix4f &prjM, int framePos)const;
    
    //switch critical
    void switchCritical(tvPoint3DSet &vtvPoint, int framePos,int fNum);
    
    //update 3D primitive using sfmData & lblPoly(contains 2D points of models)
    void updatePrimitive3D(tvPoint3DSet &vtvPoint, int framePos,
                           ivAppSfM *sfmData, lblPolygon2D &lblPoly);
    
    //rendering for current view 2D
    void renderOCV(tvPoint3DSet &vtvPoint, cv::Mat &canvas, int framePos, int selectedPt, bool isSelected);
    
    //rendering for novel view 3D
    void renderOGL(tvPoint3DSet &vtvPoint, int framePos);
    
    //rendering for optimized points
    void renderOGL(tvPoint3DSet &vtvPoint);
    
    //io
    friend std::istream& operator>>(std::istream& is, tvTriPrim& prim);
    friend std::ostream& operator<<(std::ostream& os, const tvTriPrim& prim);
};

//primitive set
typedef std::vector<tvTriPrim> tvTriPrimSet;

#endif /* defined(__ivModeling__tvTriPoly__) */
